var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
import { Injectable } from '@angular/core';
import { StorageService } from '../storage/storage.service';
import { DatabaseService } from '../storage/database.service';
import { Subject } from "rxjs";
import { AppConfiguration } from "../common/app-configuration";
import { EventsManagerService } from "../common/events-manager.service";
var TOKEN_AHEAD_EXPIRATION = 60; // seconds
export var AuthService = (function () {
    function AuthService(storage, database, eventsManagerService) {
        this.storage = storage;
        this.database = database;
        this.eventsManagerService = eventsManagerService;
        this.refresTokenTimer = -1;
        this._tokenWillExpire = new Subject();
        if (AppConfiguration.currentConfig.clientId) {
            this.setClientId(AppConfiguration.currentConfig.clientId);
        }
        if (AppConfiguration.currentConfig.clientSecret) {
            this.setClientSecret(AppConfiguration.currentConfig.clientSecret);
        }
    }
    Object.defineProperty(AuthService.prototype, "tokenWillExpire", {
        get: function () {
            return this._tokenWillExpire.asObservable();
        },
        enumerable: true,
        configurable: true
    });
    AuthService.prototype.setWebSocketsService = function (webSocketsService) {
        this.webSocketsService = webSocketsService;
    };
    AuthService.prototype.logout = function () {
        if (this.webSocketsService && this.webSocketsService.isConnected) {
            this.webSocketsService.disconnect();
        }
        this.storage.clear();
        this.database.clear(['local_gateways']);
        this.eventsManagerService.trigger(EventsManagerService.ON_USER_SIGN_OUT);
    };
    AuthService.prototype.isLoggedIn = function () {
        var loggedIn = this.getAccessToken() !== null && this.getUser() !== null;
        if (loggedIn && this.refresTokenTimer == -1) {
            var accessTokenRegistration = this.storage.get('ATR');
            var storedTokenLifetime = this.storage.get('ATL');
            if (accessTokenRegistration && storedTokenLifetime) {
                var tokenCreation = parseInt(accessTokenRegistration.toString());
                tokenCreation = Math.floor((Date.now() - tokenCreation) / 1000);
                var tokenLifetime = parseInt(storedTokenLifetime.toString());
                if (tokenCreation > tokenLifetime) {
                    this._tokenWillExpire.next(true);
                }
                else {
                    tokenLifetime = tokenLifetime - tokenCreation;
                    this.startRefreshTokenTimeout(tokenLifetime);
                }
            }
        }
        return loggedIn;
    };
    AuthService.prototype.getAccessToken = function () {
        return this.storage.get('AT');
    };
    AuthService.prototype.getRefreshToken = function () {
        return this.storage.get('RT');
    };
    AuthService.prototype.setClientId = function (clientId) {
        this.clientId = clientId;
    };
    AuthService.prototype.setClientSecret = function (clientSecret) {
        this.clientSecret = clientSecret;
    };
    AuthService.prototype.getClientId = function () {
        return this.clientId;
    };
    AuthService.prototype.getClientSecret = function () {
        return this.clientSecret;
    };
    AuthService.prototype.startRefreshTokenTimeout = function (ttl) {
        var _this = this;
        var checkExpiration = ttl < TOKEN_AHEAD_EXPIRATION ? ttl : ttl - TOKEN_AHEAD_EXPIRATION;
        if (this.refresTokenTimer) {
            clearTimeout(this.refresTokenTimer);
        }
        this.refresTokenTimer = setTimeout(function () {
            _this._tokenWillExpire.next(true);
        }, checkExpiration * 1000);
    };
    AuthService.prototype.getNewAccessTokenFromRefreshToken = function () {
        this._tokenWillExpire.next(true);
    };
    AuthService.prototype.setAccessToken = function (accessToken, ttl) {
        this.startRefreshTokenTimeout(ttl);
        this.storage.set({ key: 'ATR', value: Date.now() });
        this.storage.set({ key: 'ATL', value: ttl });
        return this.storage.set({ key: 'AT', value: accessToken, ttl: ttl });
    };
    AuthService.prototype.setRefreshToken = function (refreshToken) {
        if (refreshToken) {
            return this.storage.set({ key: 'RT', value: refreshToken });
        }
    };
    AuthService.prototype.getUser = function () {
        return this.storage.get('US');
    };
    AuthService.prototype.setUser = function (user) {
        return this.storage.set({ key: 'US', value: user });
    };
    AuthService.prototype.setAccount = function (account) {
        return this.storage.set({ key: 'AC', value: account });
    };
    AuthService.prototype.getAccount = function () {
        return this.storage.get('AC');
    };
    AuthService = __decorate([
        // seconds
        Injectable(), 
        __metadata('design:paramtypes', [StorageService, DatabaseService, EventsManagerService])
    ], AuthService);
    return AuthService;
}());
//# sourceMappingURL=auth.service.js.map