var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
import { Injectable } from '@angular/core';
import { Response, Headers, RequestOptions } from '@angular/http';
import { Subject } from 'rxjs';
import { Observable } from 'rxjs/Observable';
import 'rxjs/add/operator/catch';
import 'rxjs/add/operator/map';
import 'rxjs/add/observable/throw';
import { AuthService } from "../auth/auth.service";
import { Oauth2Service } from '../resources/oauth-2.service';
import { AppConfiguration } from "../common/app-configuration";
import { EventsManagerService } from "../common/events-manager.service";
import { HttpInterceptor } from "../common/http-interceptor.service";
export var APIClientService = (function () {
    function APIClientService(defaultPath, http, authService, eventsManagerService) {
        this.defaultPath = defaultPath;
        this.http = http;
        this.authService = authService;
        this.eventsManagerService = eventsManagerService;
        this._onServerError = new Subject();
        this.defaultPath = defaultPath ? defaultPath : '';
    }
    Object.defineProperty(APIClientService.prototype, "apiURL", {
        get: function () {
            return AppConfiguration.currentAPIURL;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(APIClientService.prototype, "onServerError", {
        get: function () {
            return this._onServerError.asObservable();
        },
        enumerable: true,
        configurable: true
    });
    APIClientService.prototype.get = function (path, data, options) {
        var _this = this;
        var headers = new Headers();
        var innerOptions = new RequestOptions({ headers: headers });
        var resourcePath = this.apiURL;
        if (this.defaultPath) {
            resourcePath += '/' + this.defaultPath;
        }
        if (!options || options.credentials === undefined || options.credentials === true) {
            headers.append('Authorization', 'Bearer ' + this.authService.getAccessToken());
        }
        if (path) {
            resourcePath += '/' + path;
        }
        if (data) {
            resourcePath = this.addUrlParams(data, resourcePath);
        }
        return this.http.get(resourcePath, innerOptions)
            .map(this.extractData)
            .catch(function (error) {
            var errorData;
            try {
                errorData = error._body ? JSON.parse(error._body) : undefined;
            }
            catch (parseError) {
                console.log('Response Parse error: ' + parseError.toString());
            }
            var errors = errorData && errorData.error && errorData.error.errors || [];
            var accessToken = _this.authService.getAccessToken();
            /**
             * code 114 means Invalid access_token
             */
            if (accessToken && errors.length && (errors[0].code === 114 || errors[0].code === 115)) {
                var oauth2Service = new Oauth2Service(_this.http, _this.authService, _this.eventsManagerService);
                return oauth2Service.refreshToken()
                    .do(function (response) { return _this.get(resourcePath, innerOptions); });
            }
            return _this.handleError(error);
        });
    };
    APIClientService.prototype.addUrlParams = function (data, resourcePath) {
        if (data) {
            var params = [];
            var keys = Object.keys(data);
            for (var i = 0; i < keys.length; i++) {
                params.push(keys[i] + '=' + encodeURIComponent(data[keys[i]]));
            }
            if (params.length > 0) {
                resourcePath += '?' + params.join('&');
            }
        }
        return resourcePath;
    };
    APIClientService.prototype.remove = function (path, data, options) {
        var _this = this;
        var headers = new Headers();
        var innerOptions = new RequestOptions({ headers: headers });
        var resourcePath = this.apiURL;
        if (this.defaultPath) {
            resourcePath += '/' + this.defaultPath;
        }
        if (!options || options.credentials === undefined || options.credentials === true) {
            headers.append('Authorization', 'Bearer ' + this.authService.getAccessToken());
        }
        if (path) {
            resourcePath += '/' + path;
        }
        if (data) {
            resourcePath = this.addUrlParams(data, resourcePath);
        }
        return this.http.delete(resourcePath, innerOptions)
            .map(this.extractData)
            .catch(function (error) {
            var errorData;
            try {
                errorData = error._body ? JSON.parse(error._body) : undefined;
            }
            catch (parseError) {
                console.log('Response Parse error: ' + parseError.toString());
            }
            var errors = errorData && errorData.error && errorData.error.errors || [];
            var accessToken = _this.authService.getAccessToken();
            /**
             * code 114 means Invalid access_token
             */
            if (accessToken && errors.length && (errors[0].code === 114 || errors[0].code === 115)) {
                var oauth2Service = new Oauth2Service(_this.http, _this.authService, _this.eventsManagerService);
                return oauth2Service.refreshToken()
                    .do(function (response) { return _this.remove(resourcePath, innerOptions); });
            }
            /**
             * Conflict case
             */
            if (error.status === 409) {
                return Observable.throw(Object.assign({}, error, {
                    data: errorData
                }));
            }
            if (error.status >= 500) {
                _this.eventsManagerService.trigger(EventsManagerService.ON_SERVER_CONNECTIVITY_ERROR, error);
            }
            return _this.handleError(error);
        });
    };
    APIClientService.prototype.post = function (path, data, options) {
        var _this = this;
        var bodyString = JSON.stringify(data);
        var headers = new Headers({ 'Content-Type': 'application/json' });
        var innerOptions = new RequestOptions({ headers: headers });
        var resourcePath = this.apiURL;
        if (this.defaultPath) {
            resourcePath += '/' + this.defaultPath;
        }
        if (path) {
            resourcePath += '/' + path;
        }
        if (!options || options.credentials === undefined || options.credentials === true) {
            headers.append('Authorization', 'Bearer ' + this.authService.getAccessToken());
        }
        if (options && options.data) {
            resourcePath = this.addUrlParams(options.data, resourcePath);
        }
        return this.http.post(resourcePath, bodyString, innerOptions)
            .map(this.extractData)
            .catch(function (error) {
            var errorData;
            try {
                errorData = error._body ? JSON.parse(error._body) : undefined;
            }
            catch (parseError) {
                console.log('Response Parse error: ' + parseError.toString());
            }
            var errors = errorData && errorData.error && errorData.error.errors || [];
            var accessToken = _this.authService.getAccessToken();
            /**
             * code 114 means Invalid access_token
             */
            if (accessToken && errors.length && (errors[0].code === 114 || errors[0].code === 115)) {
                var oauth2Service = new Oauth2Service(_this.http, _this.authService, _this.eventsManagerService);
                return oauth2Service.refreshToken()
                    .do(function (response) { return _this.post(resourcePath, innerOptions); });
            }
            if (error.status >= 500) {
                _this.eventsManagerService.trigger(EventsManagerService.ON_SERVER_CONNECTIVITY_ERROR, error);
            }
            return _this.handleError(error);
        });
    };
    APIClientService.prototype.put = function (path, data, options) {
        var _this = this;
        var bodyString = JSON.stringify(data);
        var headers = new Headers({ 'Content-Type': 'application/json' });
        var innerOptions = new RequestOptions({ headers: headers });
        var resourcePath = this.apiURL;
        if (this.defaultPath) {
            resourcePath += '/' + this.defaultPath;
        }
        if (path) {
            resourcePath += '/' + path;
        }
        if (!options || options.credentials === undefined || options.credentials === true) {
            headers.append('Authorization', 'Bearer ' + this.authService.getAccessToken());
        }
        if (options && options.data) {
            resourcePath = this.addUrlParams(options.data, resourcePath);
        }
        return this.http.put(resourcePath, bodyString, innerOptions)
            .map(this.extractData)
            .catch(function (error) {
            var errorData;
            try {
                errorData = error._body ? JSON.parse(error._body) : undefined;
            }
            catch (parseError) {
                console.log('Response Parse error: ' + parseError.toString());
            }
            var errors = errorData && errorData.error && errorData.error.errors || [];
            var accessToken = _this.authService.getAccessToken();
            /**
             * code 114 means Invalid access_token
             */
            if (accessToken && errors.length && (errors[0].code === 114 || errors[0].code === 115)) {
                var oauth2Service = new Oauth2Service(_this.http, _this.authService, _this.eventsManagerService);
                return oauth2Service.refreshToken()
                    .do(function (response) { return _this.put(resourcePath, innerOptions); });
            }
            /**
             * Conflict case
             */
            if (error.status === 409) {
                return Observable.throw(Object.assign({}, error, {
                    data: errorData
                }));
            }
            if (error.status >= 500) {
                _this.eventsManagerService.trigger(EventsManagerService.ON_SERVER_CONNECTIVITY_ERROR, error);
            }
            return _this.handleError(error);
        });
    };
    APIClientService.prototype.extractData = function (res) {
        return res.json();
    };
    APIClientService.prototype.handleError = function (error) {
        var errObj;
        if (error instanceof Response) {
            var body = error.json();
            errObj = body.error;
        }
        else {
            errObj = error.message ? { message: error.message } : { message: error.toString() };
        }
        return Observable.throw(errObj);
    };
    APIClientService = __decorate([
        Injectable(), 
        __metadata('design:paramtypes', [String, HttpInterceptor, AuthService, EventsManagerService])
    ], APIClientService);
    return APIClientService;
}());
//# sourceMappingURL=api-client.service.js.map