var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
import { Injectable } from '@angular/core';
import { Subject } from "rxjs";
import { APIClientService } from "../common/api-client.service";
import { AuthService } from "../auth/auth.service";
import { EventsManagerService } from "../common/events-manager.service";
import { DatabaseService } from "../storage/database.service";
import { HttpInterceptor } from "../common/http-interceptor.service";
var DEVICE_REFERENCE = 'device_reference';
export var DeviceReferenceService = (function (_super) {
    __extends(DeviceReferenceService, _super);
    function DeviceReferenceService(http, authService, databaseService, eventsService) {
        _super.call(this, 'devices', http, authService, eventsService);
        this.databaseService = databaseService;
        this.eventsService = eventsService;
        this._data = { version: '0.0.0' };
        this._loaded = false;
        this._onLoaded = new Subject();
    }
    Object.defineProperty(DeviceReferenceService.prototype, "version", {
        get: function () {
            return this._data.version;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DeviceReferenceService.prototype, "data", {
        get: function () {
            return this._data;
        },
        set: function (value) {
            this._data = value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DeviceReferenceService.prototype, "onLoaded", {
        get: function () {
            return this._onLoaded.asObservable();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DeviceReferenceService.prototype, "isLoaded", {
        get: function () {
            return this._loaded;
        },
        enumerable: true,
        configurable: true
    });
    DeviceReferenceService.prototype.loadFile = function () {
        var _this = this;
        this.loadLocalVersion().subscribe(function (localVersionResponse) {
            var date = new Date();
            var cacheString = "" + date.getUTCDate() + date.getUTCHours() + date.getUTCMinutes();
            _this.get('reference', { v: cacheString }).subscribe(function (referenceResponse) {
                _this.storeLocalVersion(referenceResponse);
            }, function (error) {
                _this._loaded = true;
                _this._onLoaded.next(true);
                console.log('Error loading reference file, using local version (' + _this.version + ')');
            });
        });
    };
    DeviceReferenceService.prototype.loadLocalVersion = function () {
        var _this = this;
        return this.databaseService.getCollection(DEVICE_REFERENCE).getById(DEVICE_REFERENCE).map(function (localVersion) {
            if (localVersion != null) {
                _this._data = localVersion;
            }
        });
    };
    DeviceReferenceService.prototype.storeLocalVersion = function (remoteData) {
        var _this = this;
        if (remoteData && remoteData.version && this._data && this._data.version) {
            var remoteVersion_1 = remoteData.version;
            var localVersion = this._data.version;
            if (this.versionCompare(remoteVersion_1, localVersion) >= 0) {
                this._data = this.resolveGroups(remoteData);
                this._data = this.injectTemplates(remoteData);
                this.databaseService.getCollection(DEVICE_REFERENCE).save(DEVICE_REFERENCE, remoteData).subscribe(function (storeResult) {
                    console.log('Device Reference: current (' + _this.version + ') remote (' + remoteVersion_1 + ')');
                    _this._loaded = true;
                    _this._onLoaded.next(true);
                });
            }
            else {
                console.log('Device Reference: current (' + this.version + ') remote (' + remoteVersion_1 + ')');
                this._loaded = true;
                this._onLoaded.next(true);
            }
        }
    };
    DeviceReferenceService.prototype.resolveGroups = function (data) {
        var groups = data && data.groups;
        if (groups) {
            for (var _i = 0, groups_1 = groups; _i < groups_1.length; _i++) {
                var group = groups_1[_i];
                if (group.hasOwnProperty('models') && group.hasOwnProperty('use')) {
                    for (var _a = 0, _b = group.models; _a < _b.length; _a++) {
                        var model = _b[_a];
                        // Do not override model if present
                        if (!data.models.hasOwnProperty(model)) {
                            data.models[model] = group.use;
                        }
                    }
                }
            }
        }
        return data;
    };
    DeviceReferenceService.prototype.injectTemplates = function (data) {
        var templates = data && data.templates;
        if (templates) {
            var models = data.models;
            if (models) {
                for (var model in models) {
                    for (var contextKey in models[model]) {
                        var context = models[model][contextKey];
                        if (context.hasOwnProperty('use') && templates.hasOwnProperty(context.use)) {
                            data.models[model][contextKey] = Object.assign(context, templates[context.use]);
                        }
                    }
                }
            }
        }
        return data;
    };
    DeviceReferenceService.prototype.isPositiveInteger = function (x) {
        return /^\d+$/.test(x);
    };
    DeviceReferenceService.prototype.validateParts = function (parts) {
        for (var i = 0; i < parts.length; ++i) {
            if (!this.isPositiveInteger(parts[i])) {
                return false;
            }
        }
        return true;
    };
    DeviceReferenceService.prototype.versionCompare = function (v1, v2) {
        var v1parts = v1.split('.').map(Number);
        var v2parts = v2.split('.').map(Number);
        if (!this.validateParts(v1parts) || !this.validateParts(v2parts)) {
            return NaN;
        }
        for (var i = 0; i < v1parts.length; ++i) {
            if (v2parts.length === i) {
                return 1;
            }
            if (v1parts[i] === v2parts[i]) {
                continue;
            }
            if (v1parts[i] > v2parts[i]) {
                return 1;
            }
            return -1;
        }
        if (v1parts.length != v2parts.length) {
            return -1;
        }
        return 0;
    };
    DeviceReferenceService = __decorate([
        Injectable(), 
        __metadata('design:paramtypes', [HttpInterceptor, AuthService, DatabaseService, EventsManagerService])
    ], DeviceReferenceService);
    return DeviceReferenceService;
}(APIClientService));
//# sourceMappingURL=device-reference.service.js.map