var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
import { Http, Request, RequestOptions, XHRBackend } from "@angular/http";
import { Injectable } from "@angular/core";
import { Observable, Subject } from "rxjs/Rx";
import { EventsManagerService } from "../common/events-manager.service";
import { AppConfiguration } from "../common/app-configuration";
import "rxjs/add/operator/catch";
import "rxjs/add/observable/throw";
import "rxjs/add/operator/map";
export var HttpInterceptor = (function (_super) {
    __extends(HttpInterceptor, _super);
    function HttpInterceptor(backend, options, http, eventsManagerService) {
        var _this = this;
        _super.call(this, backend, options);
        this.http = http;
        this.eventsManagerService = eventsManagerService;
        this._enqueueRequests = false;
        this.notifier = new Subject();
        this.eventsManagerService.onTokenRefreshStart.subscribe(function () {
            HttpInterceptor._enqueueRequests = true;
        });
        this.eventsManagerService.onServerConnectivityError.subscribe(function () {
            HttpInterceptor._enqueueRequests = true;
        });
        this.eventsManagerService.onConnectivityRecovered.subscribe(function () {
            HttpInterceptor._enqueueRequests = false;
            _this.drainQueue();
        });
        this.eventsManagerService.onTokenRefreshed.subscribe(function (_a) {
            var status = _a.status, newAccessToken = _a.newAccessToken;
            HttpInterceptor._enqueueRequests = false;
            if (status) {
                _this.drainQueue(newAccessToken);
            }
        });
        this.eventsManagerService.onTokenRefreshError.subscribe(function () {
            HttpInterceptor._enqueueRequests = false;
        });
    }
    Object.defineProperty(HttpInterceptor, "enqueueRequests", {
        get: function () {
            return HttpInterceptor._enqueueRequests;
        },
        enumerable: true,
        configurable: true
    });
    HttpInterceptor.prototype.drainQueue = function (newAccessToken) {
        this.notifier.next({ newAccessToken: newAccessToken, newAPIUrl: AppConfiguration.currentAPIURL });
    };
    HttpInterceptor.prototype.changeAccessTokenForRequest = function (headers, newAccessToken) {
        if (newAccessToken) {
            headers.set('authorization', newAccessToken);
        }
        return headers;
    };
    HttpInterceptor.prototype.changeAPIUrlForRequest = function (url, newAPIUrl) {
        return url.replace(/.*smartenit\.io\/v2/, newAPIUrl);
    };
    HttpInterceptor.prototype.getNextRequest = function (newAccessToken, newAPIUrl, prevRequest, options) {
        var adjustedUrl = new Request({
            method: prevRequest.method,
            url: this.changeAPIUrlForRequest(prevRequest.url, newAPIUrl),
            headers: this.changeAccessTokenForRequest(prevRequest.headers, newAccessToken),
            body: prevRequest.getBody(),
            responseType: prevRequest.responseType
        });
        return _super.prototype.request.call(this, adjustedUrl, options);
    };
    HttpInterceptor.prototype.request = function (url, options) {
        var _this = this;
        var localUrl;
        if (typeof url == 'string') {
            localUrl = url;
        }
        else {
            localUrl = url.url;
        }
        var urlToRequest;
        if (url instanceof Request) {
            urlToRequest = url;
        }
        else if (typeof url == 'string') {
            urlToRequest = new Request({ url: url });
        }
        if (HttpInterceptor._enqueueRequests && localUrl.indexOf('oauth2/token') < 0) {
            return this.notifier.asObservable()
                .flatMap(function (_a) {
                var newAccessToken = _a.newAccessToken, newAPIUrl = _a.newAPIUrl;
                var adjustedUrl = new Request({
                    method: urlToRequest.method,
                    url: _this.changeAPIUrlForRequest(urlToRequest.url, newAPIUrl),
                    search: urlToRequest.search,
                    headers: _this.changeAccessTokenForRequest(urlToRequest.headers, newAccessToken),
                    body: urlToRequest.body,
                    responseType: urlToRequest.responseType
                });
                return _super.prototype.request.call(_this, adjustedUrl, options);
            });
        }
        var self = this;
        if (localUrl.indexOf('build/') < 0) {
            return _super.prototype.request.call(this, url, options)
                .timeout(11 * 1000)
                .map(function (response) {
                if (localUrl.indexOf('oauth2/token') >= 0) {
                    var jsonResponse = response.json();
                    if (jsonResponse && jsonResponse.access_token) {
                        HttpInterceptor._enqueueRequests = false;
                        _this.notifier.next({ newAccessToken: 'Bearer ' + jsonResponse.access_token, newAPIUrl: AppConfiguration.currentAPIURL });
                    }
                }
                return response;
            })
                .catch(function (error) {
                console.log('intercepted error: ', error, url);
                if (error.name == 'TimeoutError' || error.status == 0 || error.status == 403 || error.status == 401) {
                    self.eventsManagerService.trigger(EventsManagerService.ON_SERVER_CONNECTIVITY_ERROR, { error: error, enqueueRequests: HttpInterceptor.enqueueRequests });
                    HttpInterceptor._enqueueRequests = true;
                    if (localUrl.indexOf('oauth2/token') >= 0) {
                        AppConfiguration.restoreInitialConfiguration();
                        return Observable.throw(error);
                    }
                    else if (localUrl.indexOf('devices/reference') >= 0) {
                        return Observable.throw(error);
                    }
                    else {
                        return _this.notifier.asObservable()
                            .flatMap(function (_a) {
                            var newAccessToken = _a.newAccessToken, newAPIUrl = _a.newAPIUrl;
                            var adjustedUrl = new Request({
                                method: urlToRequest.method,
                                url: _this.changeAPIUrlForRequest(urlToRequest.url, newAPIUrl),
                                search: urlToRequest.search,
                                headers: _this.changeAccessTokenForRequest(urlToRequest.headers, newAccessToken),
                                body: urlToRequest.body,
                                responseType: urlToRequest.responseType
                            });
                            return _super.prototype.request.call(_this, adjustedUrl, options);
                        });
                    }
                }
                return Observable.throw(error);
            });
        }
        else {
            return Observable.empty();
        }
    };
    HttpInterceptor._enqueueRequests = false;
    HttpInterceptor = __decorate([
        Injectable(), 
        __metadata('design:paramtypes', [XHRBackend, RequestOptions, Http, EventsManagerService])
    ], HttpInterceptor);
    return HttpInterceptor;
}(Http));
//# sourceMappingURL=http-interceptor.service.js.map