var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
import { Injectable } from '@angular/core';
import { Observable } from "rxjs";
import 'rxjs/add/operator/catch';
import 'rxjs/add/operator/map';
import 'rxjs/add/observable/throw';
import 'rxjs/add/operator/timeoutWith';
import { AuthService } from "../auth/auth.service";
import { Oauth2Service } from "../resources/oauth-2.service";
import { AppConfiguration } from "../common/app-configuration";
import { DevicesService } from "../resources/devices.service";
import { DatabaseService } from "../storage/database.service";
import { EventsManagerService } from "../common/events-manager.service";
import { HttpInterceptor } from "../common/http-interceptor.service";
export var LocalConnectionService = (function () {
    function LocalConnectionService(http, authService, oauthService, devicesService, database, eventsService) {
        var _this = this;
        this.http = http;
        this.authService = authService;
        this.oauthService = oauthService;
        this.devicesService = devicesService;
        this.database = database;
        this.eventsService = eventsService;
        this.usingLocalConnection = false;
        this.apiVersionUrl = '/v2';
        this.switchingConnection = false;
        this.eventsService.onServerConnectivityError.subscribe(function (_a) {
            var enqueueRequests = _a.enqueueRequests;
            if (enqueueRequests && !_this.switchingConnection) {
                console.log('Connectivity error, checking connection switching');
                _this.switchingConnection = true;
                _this.checkLocalConnection();
            }
        });
    }
    LocalConnectionService.prototype.startLocalConnectionTimer = function () {
        var _this = this;
        this.usingLocalConnection = true;
        setTimeout(function () {
            _this.usingLocalConnection = false;
        }, 10 * 60 * 1000);
    };
    LocalConnectionService.prototype.testConnections = function (domains, assignConnection, refreshToken) {
        var _this = this;
        if (assignConnection === void 0) { assignConnection = true; }
        if (refreshToken === void 0) { refreshToken = true; }
        var requests = [];
        var pingUrl = this.apiVersionUrl + '/ping';
        var localGateways = this.database.getCollection('local_gateways');
        var observables = [];
        domains.forEach(function (domain) {
            var path = 'https://' + domain.domain;
            var secondPath = path + ':' + 54443;
            var thirdPath = path + ':' + 54444;
            observables.push(_this.http.get(path + pingUrl)
                .timeout(2000)
                .map(function (response) { return Observable.of({ url: path, domain: domain, response: response }); })
                .catch(function () {
                return _this.http.get(secondPath + pingUrl)
                    .timeout(2000)
                    .map(function (response) { return Observable.of({ url: secondPath, domain: domain, response: response }); })
                    .catch(function () {
                    return _this.http.get(thirdPath + pingUrl)
                        .timeout(2000)
                        .map(function (response) { return Observable.of({ url: thirdPath, domain: domain, response: response }); })
                        .catch(function () {
                        return Observable.of({ error: true });
                    });
                });
            }));
        });
        var anyConnectionAssigned = false;
        var subscription = Observable.forkJoin(observables)
            .subscribe(function (val) {
            var successResponse;
            if (val && val.length > 0) {
                successResponse = val.find(function (item) { return (item && !item.error); });
                var workingRequest = successResponse;
                if (workingRequest && workingRequest.value && workingRequest.value.response) {
                    var data = workingRequest.value.response.json();
                    var versionValid = false;
                    if (data && data.version) {
                        var version = data.version.split('.');
                        if (version.length >= 1) {
                            if (Number(version[0]) == 3) {
                                if (version.length >= 2) {
                                    if (Number(version[1]) == 4) {
                                        if (version.length >= 3) {
                                            var latest = version[2].split('-');
                                            if (latest.length >= 1) {
                                                if (Number(latest[0]) >= 0) {
                                                    versionValid = true;
                                                }
                                            }
                                        }
                                    }
                                    else if (Number(version[1]) > 4) {
                                        versionValid = true;
                                    }
                                }
                            }
                            else if (Number(version[0]) > 3) {
                                versionValid = true;
                            }
                        }
                    }
                    if (!versionValid) {
                        return Observable.throw(new Error('Invalid gateway version'));
                    }
                    else {
                        if (workingRequest && workingRequest.value && workingRequest.value.url) {
                            var url = workingRequest.value.url;
                            localGateways.save(workingRequest.value.domain.domain, workingRequest.value.domain).subscribe();
                            if (assignConnection && AppConfiguration.currentConfig.useLocalConnection && !_this.usingLocalConnection) {
                                anyConnectionAssigned = true;
                                _this.setNewAPIURL(url + _this.apiVersionUrl, workingRequest.value.domain.gateway, refreshToken);
                            }
                        }
                    }
                }
            }
        }, function (error) {
            if (assignConnection && !anyConnectionAssigned) {
                console.log('Local address not available, using remote API');
                _this.switchingConnection = false;
                AppConfiguration.restoreInitialConfiguration();
                if (refreshToken) {
                    _this.authService.getNewAccessTokenFromRefreshToken();
                }
                _this.eventsService.trigger(EventsManagerService.ON_CONNECTION_SETUP);
            }
        }, function () {
            if (assignConnection && !anyConnectionAssigned) {
                console.log('Local address not available, using remote API');
                _this.switchingConnection = false;
                AppConfiguration.restoreInitialConfiguration();
                if (refreshToken) {
                    _this.authService.getNewAccessTokenFromRefreshToken();
                }
                _this.eventsService.trigger(EventsManagerService.ON_CONNECTION_SETUP);
            }
            if (subscription) {
                subscription.unsubscribe();
            }
        });
    };
    LocalConnectionService.prototype.checkRemoteConnections = function (assignRemoteConnection, testConnections, refreshToken) {
        var _this = this;
        if (assignRemoteConnection === void 0) { assignRemoteConnection = true; }
        if (testConnections === void 0) { testConnections = true; }
        if (refreshToken === void 0) { refreshToken = true; }
        var localGateways = this.database.getCollection('local_gateways');
        if (this.authService.isLoggedIn()) {
            // check remote connections
            localGateways.list().subscribe(function (urls) {
                if (!AppConfiguration.currentConfig.apiURLSet && urls && urls.length > 0) {
                    var subscription_1 = _this.eventsService.onConnectionSetup.subscribe(function () {
                        _this.checkGateways(localGateways, assignRemoteConnection, testConnections, refreshToken);
                        subscription_1.unsubscribe();
                    }, function (error) {
                        subscription_1.unsubscribe();
                    }, function () {
                        subscription_1.unsubscribe();
                    });
                }
                else {
                    _this.checkGateways(localGateways, assignRemoteConnection, testConnections, refreshToken);
                }
            });
        }
    };
    LocalConnectionService.prototype.checkGateways = function (localGateways, assignRemoteConnection, testConnections, refreshToken) {
        var _this = this;
        if (assignRemoteConnection === void 0) { assignRemoteConnection = true; }
        if (testConnections === void 0) { testConnections = true; }
        if (refreshToken === void 0) { refreshToken = true; }
        console.log('Getting Gateways...');
        var subscription = this.devicesService.listFromBackend({
            type: 'gateway',
            domain: { $nin: [null, {}, ''] }
        }).subscribe(function (gateways) {
            gateways = gateways.data;
            var domains = gateways.map(function (gateway) {
                if (gateway && gateway.domain && gateway.domain.name) {
                    return {
                        gateway: {
                            _id: gateway._id,
                            hwId: gateway.hwId
                        },
                        domain: gateway.domain.name
                    };
                }
            });
            if (domains.length > 0) {
                localGateways.clear();
                domains.forEach(function (element) {
                    localGateways.save(element.domain, element).subscribe();
                });
            }
            if (testConnections) {
                _this.testConnections(domains, assignRemoteConnection, refreshToken);
            }
            subscription.unsubscribe();
        }, function (error) {
            subscription.unsubscribe();
        }, function () {
            subscription.unsubscribe();
        });
    };
    LocalConnectionService.prototype.setNewAPIURL = function (url, gateway, refreshToken) {
        var _this = this;
        if (refreshToken === void 0) { refreshToken = true; }
        this.switchingConnection = false;
        AppConfiguration.setCurrentGateway(gateway);
        AppConfiguration.setAPIURL(url);
        this.startLocalConnectionTimer();
        if (this.authService.isLoggedIn()) {
            var subscription_2 = this.eventsService.onTokenRefreshed.subscribe(function () {
                _this.eventsService.trigger(EventsManagerService.ON_CONNECTION_SETUP, url);
                subscription_2.unsubscribe();
            });
            if (refreshToken) {
                this.authService.getNewAccessTokenFromRefreshToken();
            }
        }
        else {
            this.eventsService.trigger(EventsManagerService.ON_CONNECTION_SETUP, url);
        }
    };
    LocalConnectionService.prototype.checkLocalConnection = function (refreshToken) {
        var _this = this;
        if (refreshToken === void 0) { refreshToken = true; }
        if (AppConfiguration.currentConfig.useLocalConnection) {
            console.log('Checking Local Connections...');
            // TODO check if current local connection is OK
            var localGateways = this.database.getCollection('local_gateways');
            this.usingLocalConnection = false;
            var subscription_3 = localGateways.list().subscribe(function (urls) {
                if (urls && urls.length > 0) {
                    var domains = urls.map(function (item) { return item.value; });
                    _this.testConnections(domains, true, refreshToken);
                    if (_this.authService.isLoggedIn()) {
                        _this.checkRemoteConnections(false, false, refreshToken);
                    }
                }
                else if (_this.authService.isLoggedIn()) {
                    _this.switchingConnection = false;
                    _this.checkRemoteConnections(true, true, refreshToken);
                }
                else {
                    _this.switchingConnection = false;
                    _this.eventsService.trigger(EventsManagerService.ON_CONNECTION_SETUP);
                }
                subscription_3.unsubscribe();
            });
        }
        else {
            this.switchingConnection = false;
            this.eventsService.trigger(EventsManagerService.ON_CONNECTION_SETUP);
        }
    };
    LocalConnectionService = __decorate([
        Injectable(), 
        __metadata('design:paramtypes', [HttpInterceptor, AuthService, Oauth2Service, DevicesService, DatabaseService, EventsManagerService])
    ], LocalConnectionService);
    return LocalConnectionService;
}());
//# sourceMappingURL=local-connection.service.js.map