import { Observable } from "rxjs";
import { DevicesService } from "../resources/devices.service";
import { DatabaseService } from "../storage/database.service";
import { WebSocketsService } from "../websockets/websockets.service";
import { IWebSocketDeviceMessage } from "../websockets/websocket-device-message.interface";
import { PluginFactoryService } from "../plugins/plugin-factory.service";
import { SmartenitPlugin } from "../plugins/smartenit-plugin";
import { Model } from "../common/model";
export declare class DeviceModel extends Model {
    protected devicesService: DevicesService;
    protected dbService: DatabaseService;
    webSocketsService: WebSocketsService;
    protected pluginFactory: PluginFactoryService;
    private _plugins;
    private _type;
    private _model;
    private _processors;
    private _online;
    private _cachedData;
    private _onlineTimeout;
    private _ownerId;
    private _parents;
    private static _requests;
    private static _totalRequestCount;
    private _deviceRequests;
    private firstRequest;
    private queueInterval;
    private deviceSocketSubscription;
    readonly processors: any;
    readonly model: string;
    meta: any;
    readonly type: string;
    readonly plugins: any;
    readonly domain: any;
    readonly hwId: boolean;
    readonly online: boolean;
    cached: boolean;
    readonly ownerId: string;
    readonly parents: any;
    private _onDeviceData;
    readonly onDeviceData: Observable<IWebSocketDeviceMessage>;
    private _onDeviceOnlineState;
    readonly onDeviceOnlineState: Observable<IWebSocketDeviceMessage>;
    private _onDeviceResponse;
    readonly onDeviceResponse: Observable<any>;
    private static getPluginKey(componentId, processorName);
    constructor(data: any, devicesService: DevicesService, dbService: DatabaseService, webSocketsService: WebSocketsService, pluginFactory: PluginFactoryService);
    loadBasicInfo(deviceInstance: DeviceModel, data: any): void;
    loadPluginsAndProcessors(deviceInstance: DeviceModel, data: any): void;
    getPlugin(componentId: string, processorName: string): SmartenitPlugin;
    getStatus(componentId: string, processorName: string, payload?: any, query?: string): void;
    private innerExecuteMethod(componentId, processorName, method, payload?, requestStatus?, statusPayload?, query?);
    private drainQueue();
    private drainRequest(request, key);
    private addToQueue(componentId, processorName, method, payload?, requestStatus?, payloadStatus?, query?);
    executeMethod(componentId: string, processorName: string, method: string, payload?: any, requestStatus?: boolean, payloadStatus?: any, query?: string): void;
    getAttribute(componentId: string, processorName: string, attribute: string, payload?: any, query?: string): Observable<any>;
    setAttribute(componentId: string, processorName: string, attribute: string, value: any, readAttribute?: boolean): Observable<any>;
    getProcessor(name: string): any;
    getComponent(id: string): any;
    isChildrenOf(id: string, hwId: string): any;
    startOfflineTimeout(): void;
    clearOfflineTimeout(): void;
    clearResources(): void;
}
