var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
import { Subject } from "rxjs";
import { Model } from "../common/model";
export var DeviceModel = (function (_super) {
    __extends(DeviceModel, _super);
    function DeviceModel(data, devicesService, dbService, webSocketsService, pluginFactory) {
        var _this = this;
        _super.call(this, devicesService, data);
        this.devicesService = devicesService;
        this.dbService = dbService;
        this.webSocketsService = webSocketsService;
        this.pluginFactory = pluginFactory;
        this._onlineTimeout = null;
        this._deviceRequests = null;
        this.firstRequest = true;
        this._onDeviceData = new Subject();
        this._onDeviceResponse = new Subject();
        this._onDeviceOnlineState = new Subject();
        this._online = false;
        this._plugins = {};
        this._processors = [];
        this.loadBasicInfo(this, data);
        this.loadPluginsAndProcessors(this, data);
        this.queueInterval = setInterval(function () {
            _this.drainQueue();
        }, 2 * 1000);
    }
    Object.defineProperty(DeviceModel.prototype, "processors", {
        get: function () {
            return this._processors;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DeviceModel.prototype, "model", {
        get: function () {
            return this._model;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DeviceModel.prototype, "meta", {
        get: function () {
            return this._data.meta;
        },
        set: function (value) {
            this._data.meta = value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DeviceModel.prototype, "type", {
        get: function () {
            return this._type;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DeviceModel.prototype, "plugins", {
        get: function () {
            return this._plugins;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DeviceModel.prototype, "domain", {
        get: function () {
            return this._data.domain;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DeviceModel.prototype, "hwId", {
        get: function () {
            return this._data.hwId;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DeviceModel.prototype, "online", {
        get: function () {
            return this._online;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DeviceModel.prototype, "cached", {
        get: function () {
            return this._cachedData;
        },
        set: function (value) {
            this._cachedData = value;
            this._onDeviceOnlineState.next({ cached: this._cachedData, online: this._online });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DeviceModel.prototype, "ownerId", {
        get: function () {
            return this._ownerId;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DeviceModel.prototype, "parents", {
        get: function () {
            return this._parents;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DeviceModel.prototype, "onDeviceData", {
        get: function () {
            return this._onDeviceData.asObservable();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DeviceModel.prototype, "onDeviceOnlineState", {
        get: function () {
            return this._onDeviceOnlineState.asObservable();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DeviceModel.prototype, "onDeviceResponse", {
        get: function () {
            return this._onDeviceResponse.asObservable();
        },
        enumerable: true,
        configurable: true
    });
    DeviceModel.getPluginKey = function (componentId, processorName) {
        return ['comp', componentId, 'proc', processorName].join('_');
    };
    DeviceModel.prototype.loadBasicInfo = function (deviceInstance, data) {
        var _this = this;
        if (data && data.hasOwnProperty('_id')) {
            deviceInstance.__id = data._id;
            deviceInstance._type = data.config && data.config.type ? data.config.type : data.type;
            deviceInstance._model = data.config && data.config.model ? data.config.model : null;
            deviceInstance._ownerId = data.ownerId;
            deviceInstance._parents = data.parents;
            this.deviceSocketSubscription = deviceInstance.webSocketsService.onDeviceMessage
                .subscribe(function (message) {
                if (message.resourceId == deviceInstance._id) {
                    deviceInstance._online = true;
                    deviceInstance._onDeviceData.next(message);
                    deviceInstance._onDeviceOnlineState.next({ cached: deviceInstance.cached, online: deviceInstance.online });
                    _this.clearOfflineTimeout();
                }
            });
        }
    };
    DeviceModel.prototype.loadPluginsAndProcessors = function (deviceInstance, data) {
        deviceInstance._processors = [];
        var plugin;
        // Load plugins
        if (data.hasOwnProperty('components')) {
            var pluginKey = '';
            for (var i = 0; i < data.components.length; i++) {
                var component = data.components[i];
                for (var j = 0; j < component.processors.length; j++) {
                    var processor = component.processors[j];
                    pluginKey = DeviceModel.getPluginKey(component.id, processor.name);
                    plugin = deviceInstance.pluginFactory.createPlugin(processor.name, component.id, processor.name, deviceInstance, deviceInstance.devicesService);
                    processor.componentId = component.id;
                    deviceInstance._processors.push(processor);
                    deviceInstance._plugins[pluginKey] = plugin;
                }
            }
        }
    };
    DeviceModel.prototype.getPlugin = function (componentId, processorName) {
        var pluginKey = DeviceModel.getPluginKey(componentId, processorName);
        if (this._plugins.hasOwnProperty(pluginKey)) {
            return this._plugins[pluginKey];
        }
        return null;
    };
    DeviceModel.prototype.getStatus = function (componentId, processorName, payload, query) {
        this.executeMethod(componentId, processorName, 'status', payload, false, null, query);
    };
    DeviceModel.prototype.innerExecuteMethod = function (componentId, processorName, method, payload, requestStatus, statusPayload, query) {
        var _this = this;
        var path = [this._id, 'comps', componentId, 'procs', processorName, 'methods', method].join('/');
        if (requestStatus === undefined) {
            requestStatus = true;
        }
        if (method != 'status' && requestStatus) {
            setTimeout(function () {
                _this.getStatus(componentId, processorName, statusPayload, 'cached=false');
            }, 500);
        }
        if (query) {
            path += '?' + query;
        }
        return this.devicesService.post(path, payload);
    };
    DeviceModel.prototype.drainQueue = function () {
        if (this._deviceRequests) {
            var keys = Object.keys(this._deviceRequests);
            if (this.firstRequest && this._deviceRequests && keys.length > 0) {
                this.firstRequest = false;
                var firstRequest = this._deviceRequests[keys[0]];
                if (firstRequest) {
                    this.drainRequest(firstRequest, keys[0]);
                }
            }
            else if (this.online && keys.length > 0) {
                for (var i = 0; i < keys.length; i++) {
                    var request = this._deviceRequests[keys[i]];
                    if (request) {
                        this.drainRequest(request, keys[i]);
                    }
                }
            }
        }
    };
    DeviceModel.prototype.drainRequest = function (request, key) {
        var _this = this;
        request.subscribe(function (response) {
            _this._onDeviceResponse.next({ response: response, key: key });
        });
        delete this._deviceRequests[key];
    };
    DeviceModel.prototype.addToQueue = function (componentId, processorName, method, payload, requestStatus, payloadStatus, query) {
        var path = [componentId, processorName, 'methods', method].join('-');
        if (!this._deviceRequests) {
            this._deviceRequests = {};
        }
        if (this._deviceRequests[path]) {
            this._deviceRequests[path] = this.innerExecuteMethod(componentId, processorName, method, payload, requestStatus, payloadStatus, query);
            this.drainRequest(this._deviceRequests[path], path);
        }
        else {
            this._deviceRequests[path] = this.innerExecuteMethod(componentId, processorName, method, payload, requestStatus, payloadStatus, query);
        }
    };
    DeviceModel.prototype.executeMethod = function (componentId, processorName, method, payload, requestStatus, payloadStatus, query) {
        this.startOfflineTimeout();
        var path = [this._id, 'comps', componentId, 'procs', processorName, 'methods', method].join('/');
        if (payload === null) {
            payload = {};
        }
        this.addToQueue(componentId, processorName, method, payload, requestStatus, payloadStatus, query);
        this.drainQueue();
    };
    DeviceModel.prototype.getAttribute = function (componentId, processorName, attribute, payload, query) {
        var _this = this;
        this.startOfflineTimeout();
        var path = [this._id, 'comps', componentId, 'procs', processorName, 'attrs', attribute].join('/');
        if (query) {
            path += '?' + query;
        }
        return this.devicesService.get(path, payload)
            .map(function (response) {
            _this._onDeviceResponse.next({ response: response, key: [componentId, processorName, 'attrs', attribute].join('-') });
            return response;
        });
    };
    DeviceModel.prototype.setAttribute = function (componentId, processorName, attribute, value, readAttribute) {
        var _this = this;
        this.startOfflineTimeout();
        var path = [this._id, 'comps', componentId, 'procs', processorName, 'attrs', attribute].join('/');
        if (readAttribute === undefined) {
            readAttribute = true;
        }
        if (readAttribute) {
            setTimeout(function () {
                _this.getAttribute(componentId, processorName, attribute, null, 'cached=false').subscribe(function (response) {
                    _this._onDeviceResponse.next({ response: response, key: path });
                });
            }, 500);
        }
        return this.devicesService.put(path, value).map(function (response) {
            _this._onDeviceResponse.next({ response: response, key: path });
            return response;
        });
    };
    DeviceModel.prototype.getProcessor = function (name) {
        for (var i = 0; i < this._processors.length; i++) {
            var processor = this._processors[i];
            if (processor && processor.name === name) {
                return processor;
            }
        }
        return null;
    };
    DeviceModel.prototype.getComponent = function (id) {
        for (var i = 0; i < this._data.components.length; i++) {
            var component = this._data.components[i];
            if (component && component.id === id) {
                return component;
            }
        }
        return null;
    };
    DeviceModel.prototype.isChildrenOf = function (id, hwId) {
        return this._ownerId === hwId ||
            (this._parents && this._parents.devices && this._parents.devices.some(function (p) { return p.id === id; }));
    };
    DeviceModel.prototype.startOfflineTimeout = function () {
        var _this = this;
        if (this._onlineTimeout === null) {
            this._onlineTimeout = setTimeout(function () {
                _this._online = false;
                _this.firstRequest = true;
                _this._onDeviceOnlineState.next({ online: false });
            }, 60 * 10 * 1000);
        }
    };
    DeviceModel.prototype.clearOfflineTimeout = function () {
        if (this._onlineTimeout !== null) {
            clearTimeout(this._onlineTimeout);
            this._onlineTimeout = null;
        }
    };
    DeviceModel.prototype.clearResources = function () {
        if (this.queueInterval) {
            clearInterval(this.queueInterval);
        }
        if (this.deviceSocketSubscription) {
            this.deviceSocketSubscription.unsubscribe();
        }
    };
    DeviceModel._requests = {};
    DeviceModel._totalRequestCount = 0;
    return DeviceModel;
}(Model));
//# sourceMappingURL=device.model.js.map