var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
import { SmartenitPlugin } from '../smartenit-plugin';
import { Observable } from "rxjs";
var CACHE_TIME = 3600;
export var IASACEServerPlugin = (function (_super) {
    __extends(IASACEServerPlugin, _super);
    function IASACEServerPlugin() {
        _super.apply(this, arguments);
        this._bypassedZoneList = [];
        this._zones = [];
    }
    IASACEServerPlugin.prototype.onInit = function () {
        this.getCachedValues();
        this.getStatus();
    };
    IASACEServerPlugin.prototype.setEmergency = function (type) {
        var method;
        switch (type) {
            case 'emergency':
                method = 'Emergency';
                break;
            case 'fire':
                method = 'Fire';
                break;
            case 'panic':
                method = 'Panic';
                break;
        }
        this.device.executeMethod(this.componentId, this.processorName, method, {}, true);
    };
    /*
      *ArmMode*
      0x00 Disarm
      0x01 Arm Day/Home Zones Only // ARM stay
      0x02 Arm Night/Sleep Zones Only // ARM stay
      0x03 Arm All Zones // ARM away
    */
    IASACEServerPlugin.prototype.arm = function (mode) {
        this.executeMethod(this.componentId, this.processorName, 'Arm', { "ArmMode": mode, "ArmDisarmCode": "1234", "ZoneID": 1 }, true, null, 'cached=false');
    };
    IASACEServerPlugin.prototype.disarm = function () {
        this.executeMethod(this.componentId, this.processorName, 'Arm', { "ArmMode": 0, "ArmDisarmCode": "1234", "ZoneID": 1 }, true, null, 'cached=false');
    };
    IASACEServerPlugin.prototype.getZoneInformation = function (zoneId) {
        this.executeMethod(this.componentId, this.processorName, 'GetZoneInformation', { "ZoneID": zoneId }, true);
    };
    /**
     * Bypass command
     * @param ZoneIDArray Contains the id of the index of each Zone
     *  in the CIE's zone table eg. [1,3,3,4]
     * @param ArmDisarmCode Should be between four and eight
     *  alphanumeric characters in length.
     */
    IASACEServerPlugin.prototype.bypass = function (ZoneIDArray, ArmDisarmCode) {
        if (ZoneIDArray === void 0) { ZoneIDArray = []; }
        if (ArmDisarmCode === void 0) { ArmDisarmCode = ''; }
        var method = 'Bypass';
        var payload = { ZoneIDArray: ZoneIDArray, ArmDisarmCode: ArmDisarmCode };
        var requestStatus = true;
        this.executeMethod(this.componentId, this.processorName, method, payload, requestStatus);
    };
    IASACEServerPlugin.prototype.bypassZone = function (zoneId, armDisarmCode) {
        if (armDisarmCode === void 0) { armDisarmCode = ''; }
        var uniqueArray = [];
        var zoneIdx = this._bypassedZoneList.indexOf(zoneId);
        if (zoneIdx > -1) {
            uniqueArray = this._bypassedZoneList.filter(function (zone, idx) { return idx !== zoneIdx; });
        }
        else {
            uniqueArray = [zoneId].concat(this._bypassedZoneList);
        }
        this.bypass(uniqueArray, armDisarmCode);
    };
    IASACEServerPlugin.prototype.getZones = function () {
        if (this.device && this.device.meta && this.device.meta.zones) {
            return this.device.meta.zones;
        }
        return [];
    };
    IASACEServerPlugin.prototype.getBypassedZoneList = function () {
        var method = 'GetBypassedZoneList';
        var payload = {};
        var requestStatus = true;
        this.executeMethod(this.componentId, this.processorName, method, payload, requestStatus);
    };
    IASACEServerPlugin.prototype.getEffectOptions = function () {
        return [];
    };
    IASACEServerPlugin.prototype.getEffectAttribute = function () {
        return null;
    };
    IASACEServerPlugin.prototype.getEffectMethod = function (context) {
        return null;
    };
    IASACEServerPlugin.prototype.getConditionOptions = function () {
        return [];
    };
    IASACEServerPlugin.prototype.getConditionAttribute = function () {
        return null;
    };
    IASACEServerPlugin.prototype.getCachedValues = function (subscribe) {
        var _this = this;
        var cacheValues = [
            this.getCache('ArmResponse'),
            this.getCache('AlarmStatus'),
            this.getCache('ZoneIDArray'),
            this.getCache()
        ];
        var observable = Observable.forkJoin(cacheValues).flatMap(function (array) {
            if (array && array[0] != null) {
                _this._armResponse = array[0];
            }
            if (array && array[1] != null) {
                _this._alarmStatus = array[1];
            }
            if (array && array[2] != null) {
                _this._bypassedZoneList = array[2];
            }
            _this._zones = _this.device && _this.device.meta && _this.device.meta.zones ? _this.device.meta.zones : [];
            if (array && array[3] != null) {
                _this.state = array[3];
            }
            if (_this.state != null) {
                _this.device.cached = true;
            }
            _this._onUpdate.next({
                state: _this.state,
                armResponse: _this._armResponse,
                alarmStatus: _this._alarmStatus,
                bypassedZoneList: _this._bypassedZoneList,
                zones: _this._zones
            });
            return Observable.of(_this.state);
        });
        if (subscribe === true) {
            return observable;
        }
        var subscription = observable.subscribe(function () { return subscription.unsubscribe(); });
    };
    /*
      *PanelStatus*
      0x00 Panel disarmed (all zones disarmed) and ready to arm
      0x01 Armed stay
      0x02 Armed night
      0x03 Armed away
      0x04 Exit delay
      0x05 Entry delay
      0x06 Not ready to arm
      0x07 In alarm
      0x08 Arming Stay
      0x09 Arming Night
      0x0a Arming Away
    */
    /*
      *ArmNotification*:
      0x00 All Zones Disarmed
      0x01 Only Day/Home Zones Armed
      0x02 Only Night/Sleep Zones Armed
      0x03 All Zones Armed
      0x04 Invalid Arm/Disarm Code
      0x05 Not ready to arm*
      0x06 Already disarmed
    */
    // { "AlarmStatus" : "0x01", "AudibleNotification" : "0x01", "PanelStatus" : "0x03", "SecondsRemaining" : 0 }
    IASACEServerPlugin.prototype.processMessage = function (message) {
        var attributeOrMethod = message && message.attributeOrMethod;
        var response = message && message.data && message.data.response;
        if (attributeOrMethod === 'GetPanelStatusResponse' ||
            attributeOrMethod === 'PanelStatusChanged') {
            this.state = parseInt(response.value.PanelStatus);
            this._alarmStatus = parseInt(response.value.AlarmStatus);
            this.setCache('AlarmStatus', this._alarmStatus, CACHE_TIME);
            this._onUpdate.next({ state: this.state, alarmStatus: this._alarmStatus });
        }
        else if (attributeOrMethod == 'ArmResponse') {
            this._armResponse = response.value.ArmNotification;
            this.setCache('ArmResponse', this._armResponse, CACHE_TIME);
            this._onUpdate.next({ armResponse: this._armResponse });
        }
        else if (attributeOrMethod === 'SetBypassedZoneList') {
            this._bypassedZoneList = response.value.ZoneIDArray;
            this.updateZonesBypassed(this._bypassedZoneList);
            this.setCache('ZoneIDArray', this._bypassedZoneList, CACHE_TIME);
            this._onUpdate.next({ bypassedZoneList: this._bypassedZoneList });
        }
        else if (attributeOrMethod === 'BypassResponse') {
        }
        else if (attributeOrMethod === 'ZoneMap') {
            this.parseZoneMap(response.value);
        }
        return this.state;
    };
    IASACEServerPlugin.prototype.getStatusPayload = function () {
        return ['GetPanelStatusResponse', 'SetBypassedZoneList', 'ZoneMap', 'PanelStatusChanged'];
    };
    IASACEServerPlugin.prototype.getStatus = function (context, force, asObservable, payload) {
        _super.prototype.getStatus.call(this, context, true, asObservable, payload);
    };
    IASACEServerPlugin.prototype.arraysOfNumbersAreEqual = function (array1, array2) {
        if (array1 === void 0) { array1 = []; }
        if (array2 === void 0) { array2 = []; }
        if (array1.length !== array2.length) {
            return false;
        }
        array1.sort();
        array2.sort();
        var i = 0;
        for (; i < array2.length; i += 1) {
            if (parseInt(array1[i]) !== parseInt(array2[i])) {
                return false;
            }
        }
        return true;
    };
    IASACEServerPlugin.prototype.parseZoneMap = function (response) {
        var _this = this;
        var zonesMap = [];
        Object.keys(response).sort().forEach(function (key) {
            zonesMap.push(response[key]);
        });
        zonesMap = zonesMap.filter(function (zone) {
            return zone.hasOwnProperty('ZoneID') && zone.hasOwnProperty('IEEEaddress') && zone.hasOwnProperty('ZoneType');
        });
        var requestDevices = false;
        zonesMap = zonesMap.map(function (zone) {
            var zoneMap = {
                zoneId: zone["ZoneID"],
                hwId: zone["IEEEaddress"].replace(/0x/g, '').toLowerCase(),
                zoneType: zone["ZoneType"].toLowerCase(),
                bypassed: _this._bypassedZoneList.indexOf(zone["ZoneID"]) > -1
            };
            var currentZone = _this._zones.find(function (oldZone) {
                return oldZone.deviceId && oldZone.zoneId == zoneMap.zoneId && oldZone.hwId == zoneMap.hwId;
            });
            requestDevices = requestDevices || (currentZone ? false : true);
            zoneMap.name = currentZone ? currentZone.name : 'unknown';
            zoneMap.deviceId = currentZone ? currentZone.deviceId : 'unknown';
            return zoneMap;
        });
        if (this._zones.length != zonesMap.length && zonesMap.length > 0) {
            requestDevices = true;
        }
        this._zones = zonesMap;
        var unkownDeviceIds = this._zones.map(function (zone) { return zone.hwId; });
        if (!requestDevices || unkownDeviceIds.length === 0) {
            return;
        }
        // Get deviceId by HardwareId
        // TODO (jjescof): Pagination issue
        var query = { 'hwId': { '$in': unkownDeviceIds } };
        this.devicesService.list(query)
            .flatMap(function (response) { return Observable.of(response.data); })
            .flatMap(function (devices) {
            return Observable.of(devices)
                .map(function (devices) {
                return devices.map(function (device) {
                    return { deviceId: device._id, hwId: device.hwId, name: device.name };
                });
            });
        })
            .flatMap(function (devices) {
            devices.forEach(function (device) {
                var zoneIndex = _this._zones.findIndex(function (zone) {
                    return zone.hwId == device.hwId;
                });
                if (zoneIndex > -1) {
                    var zone = _this._zones[zoneIndex];
                    var newZone = Object.assign({}, zone, { name: device.name, deviceId: device.deviceId });
                    _this._zones[zoneIndex] = newZone;
                }
            });
            _this.device.meta = _this.device.meta || {};
            _this.device.meta.zones = _this._zones;
            return _this.device.save();
        })
            .subscribe(function () {
            // TODO (jjescof) zoneStatus is reading from the iaszone directly, should it be parsed here?
            _this._onUpdate.next({
                zones: _this._zones
            });
        });
    };
    IASACEServerPlugin.prototype.updateZonesBypassed = function (bypassList) {
        this._zones.forEach(function (zone) {
            zone.bypassed = bypassList.indexOf(zone.zoneId) > -1;
        });
        this.device.meta = this.device.meta || {};
        this.device.meta.zones = this._zones;
        this.device.save();
        this._onUpdate.next({
            zones: this._zones
        });
    };
    return IASACEServerPlugin;
}(SmartenitPlugin));
//# sourceMappingURL=iasace-server.plugin.js.map