var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
import { SmartenitPlugin } from "../smartenit-plugin";
import { Observable } from "rxjs";
var CACHE_TIME = 3600;
export var IASWDServerPlugin = (function (_super) {
    __extends(IASWDServerPlugin, _super);
    function IASWDServerPlugin() {
        _super.apply(this, arguments);
    }
    IASWDServerPlugin.prototype.onInit = function () {
        this.getCacheZoneValues();
    };
    Object.defineProperty(IASWDServerPlugin.prototype, "maxDuration", {
        get: function () {
            return this._maxDuration;
        },
        enumerable: true,
        configurable: true
    });
    IASWDServerPlugin.prototype.startWarning = function (duration) {
        if (duration === void 0) { duration = 240; }
        var payload = { Mode: "0x71", Duration: duration, StrobeDutyCycle: 50, StrobeLevel: "0x01" };
        this.executeMethod(this.componentId, this.processorName, 'Startwarning', payload);
    };
    IASWDServerPlugin.prototype.stopWarning = function () {
        var payload = { Mode: "0x00", Duration: 0, StrobeDutyCycle: 0, StrobeLevel: "0x00" };
        this.executeMethod(this.componentId, this.processorName, 'Startwarning', payload);
    };
    IASWDServerPlugin.prototype.getStatusPayload = function () {
        return ['MaxDuration'];
    };
    IASWDServerPlugin.prototype.processMessage = function (message) {
        var attribute = message && message.attributeOrMethod;
        var response = message && message.data && message.data.response;
        if (attribute === 'MaxDuration') {
            this._maxDuration = response.value;
            this.setCache('MaxDuration', this._maxDuration, CACHE_TIME);
            this._onUpdate.next({ maxDuration: this._maxDuration });
        }
        return this.state;
    };
    IASWDServerPlugin.prototype.getCacheZoneValues = function () {
        var _this = this;
        var cacheValues = [
            this.getCache('MaxDuration'),
            this.getCache()
        ];
        Observable.forkJoin(cacheValues).subscribe(function (array) {
            if (array && array[0] != null) {
                _this._maxDuration = array[0];
            }
            else {
                _this._maxDuration = 240;
            }
            if (array && array[1] != null) {
                _this.state = array[1];
            }
            else {
                _this.state = {};
            }
            _this._onUpdate.next({
                maxDuration: _this._maxDuration,
                state: _this.state,
            });
        });
    };
    IASWDServerPlugin.prototype.getMaxDuration = function () {
        this.getAttributeWithSubscribeOption(this.componentId, this.processorName, 'MaxDuration', {}, false);
    };
    IASWDServerPlugin.prototype.getConditionOptions = function () {
        return null;
    };
    IASWDServerPlugin.prototype.getConditionAttribute = function () {
        return null;
    };
    IASWDServerPlugin.prototype.getEffectOptions = function () {
        return [{
                name: 'Start Warning',
                value: {
                    method: 'Startwarning',
                    payload: {
                        Mode: "0x71",
                        Duration: 240,
                        StrobeDutyCycle: 50,
                        StrobeLevel: "0x01"
                    }
                },
                requireDuration: true,
                parameterKey: 'Duration'
            },
            {
                name: 'Stop Warning',
                value: {
                    method: 'Startwarning',
                    payload: {
                        Mode: "0x00",
                        Duration: 0,
                        StrobeDutyCycle: 0,
                        StrobeLevel: "0x00"
                    }
                },
                requireDuration: false,
                parameterKey: 'Duration'
            }];
    };
    IASWDServerPlugin.prototype.getEffectAttribute = function () {
        return null;
    };
    IASWDServerPlugin.prototype.getEffectMethod = function (context) {
        return null;
    };
    IASWDServerPlugin.prototype.isEffectActive = function (option, payload) {
        var mode = payload['Mode'] || "0xff";
        return ((option.name == 'Stop Warning' && mode == "0x00") || (option.name == 'Start Warning' && mode == "0x71"));
    };
    return IASWDServerPlugin;
}(SmartenitPlugin));
//# sourceMappingURL=iaswd-server.plugin.js.map