var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
import { SmartenitPlugin } from "../smartenit-plugin";
import { Observable } from "rxjs";
var CACHE_TIME = -1;
export var IASZoneServerPlugin = (function (_super) {
    __extends(IASZoneServerPlugin, _super);
    function IASZoneServerPlugin() {
        _super.apply(this, arguments);
    }
    IASZoneServerPlugin.prototype.onInit = function () {
        this.getCacheZoneValues();
    };
    Object.defineProperty(IASZoneServerPlugin.prototype, "type", {
        get: function () {
            return this._type;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IASZoneServerPlugin.prototype, "cieParent", {
        get: function () {
            return this._cieParent;
        },
        enumerable: true,
        configurable: true
    });
    IASZoneServerPlugin.prototype.belongsToPanel = function (panelHwId) {
        return panelHwId === this._cieParent;
    };
    IASZoneServerPlugin.prototype.setType = function (value) {
        if (IASZoneServerPlugin.TYPES.hasOwnProperty(value)) {
            this._type = IASZoneServerPlugin.TYPES[value];
            this.setCache('type', this._type, CACHE_TIME);
        }
    };
    IASZoneServerPlugin.prototype.getStateMessage = function (context) {
        if (context === void 0) { context = null; }
        var type = this._type && this._type.state ? this._type.state : { activeMessage: 'Active', idleMessage: 'Idle' };
        return this.isActive() ? type.activeMessage : type.idleMessage;
    };
    Object.defineProperty(IASZoneServerPlugin.prototype, "zoneId", {
        get: function () {
            return this._zoneId;
        },
        enumerable: true,
        configurable: true
    });
    IASZoneServerPlugin.prototype.isActive = function () {
        return ((this.state && this.state.alarm1 && this.state.alarm1.value)
            || (this.state && this.state.alarm2 && this.state.alarm2.value));
    };
    IASZoneServerPlugin.prototype.isEnrolled = function () {
        return this._enrolled;
    };
    IASZoneServerPlugin.prototype.updatedAt = function () {
        return this.state && this.state.sensed_at ? this.state.sensed_at : new Date();
    };
    IASZoneServerPlugin.prototype.parseStatus = function (status) {
        var _status = 0xffff & status;
        var mask = 0x0001;
        var rsp = {
            alarm1: {
                value: _status & mask,
                string: _status & mask ? "opened/alarmed" : "closed/not alarmed"
            },
            alarm2: {
                value: (_status >> 1) & mask,
                string: (_status >> 1) & mask ? "opened/alarmed" : "closed/not alarmed"
            },
            tamper: {
                value: (_status >> 2) & mask,
                string: (_status >> 2) & mask ? "Tampered" : "Not tampered"
            },
            battery: {
                value: (_status >> 3) & mask,
                string: (_status >> 3) & mask ? "Low battery" : "Battery Ok"
            },
            trouble: {
                value: (_status >> 6) & mask,
                string: (_status >> 6) & mask ? "Trouble/Failure" : "OK"
            },
            ac_main: {
                value: (_status >> 7) & mask,
                string: (_status >> 7) & mask ? "AC/Mains fault" : "AC/Mains OK"
            },
            test: {
                value: (_status >> 8) & mask,
                string: (_status >> 8) & mask ? "Sensor is in test mode" : "Sensor is in operation mode"
            },
            battery_defect: {
                value: (_status >> 9) & mask,
                string: (_status >> 9) & mask ? "Sensor detects a defective battery" : "Sensor battery is functioning normally"
            },
            sensed_at: new Date()
        };
        return rsp;
    };
    IASZoneServerPlugin.prototype.getStatusPayload = function () {
        return ['ZoneStatusChangeNotification'];
    };
    IASZoneServerPlugin.prototype.getStatus = function (context, force, asObservable, payload) {
        _super.prototype.getStatus.call(this, context, true, asObservable, payload);
    };
    IASZoneServerPlugin.prototype.processMessage = function (message) {
        var attribute = message && message.attributeOrMethod;
        var response = message && message.data && message.data.response;
        var value = response && response.value;
        if (attribute === 'ZoneStatus') {
            this.state = this.parseStatus(parseInt(response.value));
            this.saveState(this.state, CACHE_TIME);
            this._onUpdate.next({ state: this.state });
        }
        else if (attribute === 'ZoneStatusChangeNotification') {
            this.state = this.parseStatus(parseInt(response.value['ZoneStatus']));
            this.saveState(this.state, CACHE_TIME);
            this._onUpdate.next({ state: this.state });
        }
        else if (attribute === 'ZoneType' && IASZoneServerPlugin.TYPES.hasOwnProperty(response.value)) {
            this._type = IASZoneServerPlugin.TYPES[response.value];
            this.setCache('type', this._type, CACHE_TIME);
            this._onUpdate.next({ type: this._type });
        }
        else if (attribute === 'ZoneID') {
            this._zoneId = response.value;
            this.setCache('zoneId', this._zoneId, CACHE_TIME);
            this._onUpdate.next({ zoneId: this._zoneId });
        }
        else if (attribute === 'IAS_CIE_Address') {
            this._cieParent = 'IASACE-' + response.value.replace(/0x/g, '').toLowerCase();
            this.setCache('cieParent', this._cieParent, CACHE_TIME);
            this._onUpdate.next({ cieParent: this._cieParent });
        }
        else if (attribute === 'ZoneState') {
            this._enrolled = response.value === "0x01" ? true : false;
            this.setCache('enrolled', this._enrolled, CACHE_TIME);
            this._onUpdate.next({ enrolled: this._enrolled });
        }
        return this.state;
    };
    IASZoneServerPlugin.prototype.getCacheZoneValues = function () {
        var _this = this;
        var cacheValues = [
            this.getCache('zoneId'),
            this.getCache('type'),
            this.getCache('cieParent'),
            this.getCache('enrolled'),
            this.getCache()
        ];
        Observable.forkJoin(cacheValues).subscribe(function (array) {
            if (array && array[0] != null) {
                _this._zoneId = array[0];
            }
            else {
                _this._zoneId = 0;
            }
            if (array && array[1] != null) {
                _this._type = array[1];
            }
            else {
                _this._type = {};
            }
            if (array && array[2] != null) {
                _this._cieParent = array[2];
            }
            else {
                _this._cieParent = "";
            }
            if (array && array[3] != null) {
                _this._enrolled = array[3];
            }
            else {
                _this._enrolled = false;
            }
            if (array && array[4] != null) {
                _this.state = array[4];
            }
            else {
                _this.state = {};
            }
            _this._onUpdate.next({
                zoneId: _this._zoneId,
                type: _this._type,
                cieParent: _this._cieParent,
                enrolled: _this._enrolled,
                state: _this.state,
            });
        });
    };
    IASZoneServerPlugin.prototype.getZoneType = function (asObservable) {
        if (asObservable) {
            var payload = {};
            var query = 'timeout=3';
            return this.getAttributeWithSubscribeOption(this.componentId, this.processorName, 'ZoneType', payload, true, query);
        }
        else {
            this.getAttributeWithSubscribeOption(this.componentId, this.processorName, 'ZoneType', {}, false);
        }
    };
    IASZoneServerPlugin.prototype.getZoneId = function (asObservable) {
        if (asObservable) {
            var payload = {};
            var query = 'timeout=3';
            return this.getAttributeWithSubscribeOption(this.componentId, this.processorName, 'ZoneID', payload, true, query);
        }
        else {
            this.getAttributeWithSubscribeOption(this.componentId, this.processorName, 'ZoneID', {}, false);
        }
    };
    IASZoneServerPlugin.prototype.getZoneStatus = function (asObservable) {
        // const attribute = 'ZoneStatus';
        var attribute = 'ZoneStatusChangeNotification';
        if (asObservable) {
            var payload = {};
            var query = 'timeout=3';
            return this.getAttributeWithSubscribeOption(this.componentId, this.processorName, attribute, payload, true, query);
        }
        else {
            this.getAttributeWithSubscribeOption(this.componentId, this.processorName, attribute, {}, false);
        }
    };
    IASZoneServerPlugin.prototype.getCIE = function (asObservable) {
        if (asObservable) {
            var payload = {};
            var query = 'timeout=3';
            return this.getAttributeWithSubscribeOption(this.componentId, this.processorName, 'IAS_CIE_Address', payload, true, query);
        }
        else {
            this.getAttributeWithSubscribeOption(this.componentId, this.processorName, 'IAS_CIE_Address', {}, false);
        }
    };
    IASZoneServerPlugin.prototype.getEnrolledState = function (asObservable) {
        if (asObservable) {
            var payload = {};
            var query = 'timeout=3';
            return this.getAttributeWithSubscribeOption(this.componentId, this.processorName, 'ZoneState', payload, true, query);
        }
        else {
            this.getAttributeWithSubscribeOption(this.componentId, this.processorName, 'ZoneState', {}, false);
        }
    };
    IASZoneServerPlugin.prototype.getConditionOptions = function (context) {
        if (context === void 0) { context = null; }
        if (context && context.deviceType) {
            this.setType(context.deviceType);
        }
        return [
            {
                name: this._type && this._type.condition && this._type.condition.activeMessage ? this._type.condition.activeMessage : 'Alarmed',
                method: {
                    name: 'ZoneStatusChangeNotification',
                    attribute: 'ZoneStatus',
                },
                value: { '#bitmask': 3 }
            },
            {
                name: this._type && this._type.condition && this._type.condition.idleMessage ? this._type.condition.idleMessage : 'Idle',
                method: {
                    name: 'ZoneStatusChangeNotification',
                    attribute: 'ZoneStatus',
                },
                value: { '#bitmask': 3, '#expected': 0 }
            }
        ];
    };
    IASZoneServerPlugin.prototype.getConditionAttribute = function () {
        return null;
    };
    IASZoneServerPlugin.TYPES = {
        '0x0000': {
            type: "Standard CIE",
            alarm1: "System Alarm",
            alarm2: ""
        },
        '0x000d': {
            type: "Motion sensor",
            condition: {
                activeMessage: "Motion",
                idleMessage: "No Motion"
            },
            state: {
                activeMessage: "MOTION",
                idleMessage: "NO MOTION"
            }
        },
        '0x0015': {
            type: "Contact switch",
            condition: {
                activeMessage: "is Open",
                idleMessage: "is Closed"
            },
            state: {
                activeMessage: "OPEN",
                idleMessage: "CLOSED"
            }
        },
        '0x0028': {
            type: "Fire sensor",
            condition: {
                activeMessage: "Fire Detected",
                idleMessage: "No Fire"
            },
            state: {
                activeMessage: "FIRE DETECTED",
                idleMessage: "NO FIRE"
            }
        },
        '0x002a': {
            type: "Water sensor",
            condition: {
                activeMessage: "Leak Detected",
                idleMessage: "No Leak"
            },
            state: {
                activeMessage: "LEAK",
                idleMessage: "NO LEAK"
            },
            bitmask: 3
        },
        '0x002b': {
            type: "Carbon Monoxide (CO) sensor",
            condition: {
                activeMessage: "CO Detected",
                idleMessage: "No CO"
            },
            state: {
                activeMessage: "CO DETECTED",
                idleMessage: "NO CO"
            }
        },
        '0x002c': {
            type: "Personal emergency device",
            condition: {
                activeMessage: "Emergency Detected",
                idleMessage: "No Emergency"
            },
            state: {
                activeMessage: "EMERGENCY",
                idleMessage: "NO EMERGENCY"
            }
        },
        '0x002d': {
            type: "Vibration/Movement sensor",
            condition: {
                activeMessage: "Vibration Detected",
                idleMessage: "No Vibration"
            },
            state: {
                activeMessage: "VIBRATION DETECTED",
                idleMessage: "NO VIBRATION"
            }
        },
        '0x010f': {
            type: "Remote control",
            alarm1: "Panic",
            alarm2: "Emergency"
        },
        '0x0115': {
            type: "Key fob",
            alarm1: "Panic",
            alarm2: "Emergency"
        },
        '0x021d': {
            type: "Keypad",
            alarm1: "Panic",
            alarm2: "Emergency"
        },
        '0x0225': {
            type: "Standard warning device",
            alarm1: "",
            alarm2: ""
        },
        '0x0226': {
            type: "Glass break sensor",
            condition: {
                activeMessage: "Glass Break Detected",
                idleMessage: "No Glass Break"
            },
            state: {
                activeMessage: "GLASS BREAK DETECTED",
                idleMessage: "NO GLASS BREAK"
            }
        },
        '0x0229': {
            type: "Security repeater",
            alarm1: "",
            alarm2: ""
        }
    };
    return IASZoneServerPlugin;
}(SmartenitPlugin));
//# sourceMappingURL=iaszone-server.plugin.js.map