var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
import { Injectable } from "@angular/core";
import { DatabaseService } from "../storage/database.service";
import { OnOffPlugin } from "./on-off/on-off.plugin";
import { SimpleMeteringServerPlugin } from "./simple-metering-server/simple-metering-server.plugin";
import { ElectricalMeasurementServerPlugin } from "./electrical-measurement-server/electrical-measurement-server.plugin";
import { DoorLockPlugin } from "./door-lock-server/door-lock.plugin";
import { LevelControlServerPlugin } from "./level-control-server/level-control-server.plugin";
import { DiscoverPlugin } from "./discover/discover.plugin";
import { BasicServerPlugin } from "./basic-server/basic-server.plugin";
import { OnOffArrayServerPlugin } from "./on-off-array-server/on-off-array-server.plugin";
import { ThermostatServerPlugin } from './thermostat-server/thermostat-server.plugin';
import { OnOffClientPlugin } from "./on-off-client/on-off-client.plugin";
import { IASZoneServerPlugin } from "./iaszone-server/iaszone-server.plugin";
import { IASWDServerPlugin } from "./iaswd-server/iaswd-server.plugin";
import { IASWDClientPlugin } from "./iaswd-client/iaswd-client.plugin";
import { IASACEServerPlugin } from "./iasace-server/iasace-server.plugin";
import { NotificationServerPlugin } from "./notification-server/notification-server.plugin";
import { PowerConfigurationServerPlugin } from "./power-configuration-server/power-configuration-server.plugin";
import { TemperatureMeasurementServerPlugin } from "./temperature-measurement-server/temperature-measurement-server.plugin";
import { RelativeHumidityMeasurementServerPlugin } from "./relative-humidity-measurement-server/relative-humidity-measurement-server.plugin";
import { ActionsService } from "../resources/actions.service";
import { EffectsService } from "../resources/effects.service";
import { ConditionsService } from "../resources/conditions.service";
import { PluginUtilsService } from "./plugin-utils.service";
var plugins = {
    'OnOffPlugin': OnOffPlugin,
    'OnOffClientPlugin': OnOffClientPlugin,
    'SimpleMeteringServerPlugin': SimpleMeteringServerPlugin,
    'LevelControlServerPlugin': LevelControlServerPlugin,
    'DiscoverPlugin': DiscoverPlugin,
    'BasicServerPlugin': BasicServerPlugin,
    'OnOffArrayServerPlugin': OnOffArrayServerPlugin,
    'ThermostatServerPlugin': ThermostatServerPlugin,
    'ElectricalMeasurementServerPlugin': ElectricalMeasurementServerPlugin,
    'IASZoneServerPlugin': IASZoneServerPlugin,
    'DoorLockServerPlugin': DoorLockPlugin,
    'PowerConfigurationServerPlugin': PowerConfigurationServerPlugin,
    'TemperatureMeasurementServerPlugin': TemperatureMeasurementServerPlugin,
    'RelativeHumidityMeasurementServerPlugin': RelativeHumidityMeasurementServerPlugin,
    'IASWDServerPlugin': IASWDServerPlugin,
    'IASACEServerPlugin': IASACEServerPlugin,
    'IASWDClientPlugin': IASWDClientPlugin,
    'NotificationServerPlugin': NotificationServerPlugin
};
export var PluginFactoryService = (function () {
    function PluginFactoryService(dbService, actionsService, effectsService, conditionsService, pluginUtilsService) {
        this.dbService = dbService;
        this.actionsService = actionsService;
        this.effectsService = effectsService;
        this.conditionsService = conditionsService;
        this.pluginUtilsService = pluginUtilsService;
        this._plugins = {};
    }
    PluginFactoryService.getPluginKey = function (deviceId, componentId, processorName) {
        return [deviceId, 'comp', componentId, 'proc', processorName].join('_');
    };
    PluginFactoryService.prototype.createPlugin = function (pluginName, componentId, processorName, device, devicesService) {
        var key = PluginFactoryService.getPluginKey(device._id, componentId, processorName);
        if (this._plugins.hasOwnProperty(key)) {
            return this._plugins[key];
        }
        else {
            if (plugins.hasOwnProperty(pluginName + 'Plugin')) {
                var pluginClass = plugins[pluginName + 'Plugin'];
                var newPlugin = new pluginClass(pluginName, componentId, processorName, device, this.dbService, devicesService, this.actionsService, this.conditionsService, this.effectsService, this.pluginUtilsService);
                this._plugins[key] = newPlugin;
                return newPlugin;
            }
        }
        return null;
    };
    PluginFactoryService.prototype.removePluginsForDevice = function (deviceId) {
        var _this = this;
        this._plugins = Object.keys(this._plugins)
            .filter(function (key) { return !key.startsWith(deviceId); })
            .reduce(function (obj, key) {
            obj[key] = _this._plugins[key];
            return obj;
        }, {});
    };
    PluginFactoryService.prototype.clearPlugins = function () {
        this._plugins = {};
    };
    PluginFactoryService = __decorate([
        Injectable(), 
        __metadata('design:paramtypes', [DatabaseService, ActionsService, EffectsService, ConditionsService, PluginUtilsService])
    ], PluginFactoryService);
    return PluginFactoryService;
}());
//# sourceMappingURL=plugin-factory.service.js.map