var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
import { SmartenitPlugin } from "../smartenit-plugin";
import { Observable } from "rxjs";
export var SimpleMeteringServerPlugin = (function (_super) {
    __extends(SimpleMeteringServerPlugin, _super);
    function SimpleMeteringServerPlugin() {
        _super.apply(this, arguments);
    }
    SimpleMeteringServerPlugin.prototype.onInit = function () {
        var _this = this;
        if (this.device && this.device.type) {
            var isRainBee = this.device.model && this.device.model.indexOf('rainbee') > -1;
            //const isSocket = device.type.indexOf('zbmskt1') > -1;
            if (isRainBee) {
                this.state.firstUnit = SimpleMeteringServerPlugin.UNITS['0x04'][0];
                this.state.secondUnit = SimpleMeteringServerPlugin.UNITS['0x04'][1];
            }
            else {
                this.state.firstUnit = SimpleMeteringServerPlugin.UNITS['0x00'][0];
                this.state.secondUnit = SimpleMeteringServerPlugin.UNITS['0x00'][1];
            }
        }
        this.getCache('unit_of_measure').subscribe(function (units) {
            if (units && units.length > 1) {
                _this.state.firstUnit = units[0];
                _this.state.secondUnit = units[1];
            }
        });
    };
    SimpleMeteringServerPlugin.prototype.getFirstUnit = function () {
        return this.state.firstUnit;
    };
    SimpleMeteringServerPlugin.prototype.getSecondUnit = function () {
        return this.state.secondUnit;
    };
    SimpleMeteringServerPlugin.prototype.getCurrentSummation = function () {
        return this.state.currentSummation;
    };
    SimpleMeteringServerPlugin.prototype.getInstantaneousDemand = function () {
        return this.state.instantaneousPower;
    };
    SimpleMeteringServerPlugin.prototype.getUnit = function (attribute) {
        var response = '';
        switch (attribute) {
            case 'InstantaneousDemandValue':
                response = this.getFirstUnit();
                break;
            case 'CurrentSummationDeliveredValue':
                response = this.getSecondUnit();
                break;
        }
        return response;
    };
    SimpleMeteringServerPlugin.prototype.getValue = function (attribute) {
        var response = 0;
        switch (attribute) {
            case 'CurrentSummationDeliveredValue':
                response = Number(this.state.currentSummation);
                break;
            case 'InstantaneousDemandValue':
                response = Number(this.state.instantaneousPower);
                break;
        }
        return response;
    };
    SimpleMeteringServerPlugin.prototype.setValue = function (value, attribute, subscribe) {
        // Do nothing becase values are read only
        if (subscribe) {
            return Observable.of(true);
        }
        return true;
    };
    SimpleMeteringServerPlugin.prototype.getConditionAttribute = function () {
        // return this by default but client can choose the attribute from JSON
        return 'CurrentSummationDeliveredValue';
    };
    SimpleMeteringServerPlugin.prototype.getEffectAttribute = function () {
        return null;
    };
    SimpleMeteringServerPlugin.prototype.getEffectMethod = function (context) {
        return null;
    };
    SimpleMeteringServerPlugin.prototype.getTextValue = function (context) {
        if (context && context == 'displayCurrentSummationOnly') {
            return this.getCurrentSummation() + this.getSecondUnit();
        }
        else {
            return this.getCurrentSummation() + this.getSecondUnit() + ' / ' +
                this.getInstantaneousDemand() + this.getFirstUnit();
        }
    };
    SimpleMeteringServerPlugin.prototype.getStatusPayload = function () {
        return ['CurrentSummationDeliveredValue', 'InstantaneousDemandValue', 'UnitofMeasure'];
    };
    SimpleMeteringServerPlugin.prototype.processMessage = function (message) {
        var response = message && message.data && message.data.response;
        var attribute = message && message.attributeOrMethod;
        if (attribute === 'CurrentSummationDeliveredValue') {
            this.state.currentSummation = response.value;
            this._onUpdate.next({ state: this.state });
        }
        else if (attribute === 'InstantaneousDemandValue') {
            this.state.instantaneousPower = response.value;
            this._onUpdate.next({ state: this.state });
        }
        else if (attribute === 'UnitofMeasure') {
            var units = this.parseUnits(response.value);
            this.setCache('unit_of_measure', units, 3600);
            this.state.firstUnit = units[0];
            this.state.secondUnit = units[1];
            this._onUpdate.next({ state: this.state });
        }
        return this.state;
    };
    SimpleMeteringServerPlugin.prototype.parseUnits = function (unitCode) {
        if (SimpleMeteringServerPlugin.UNITS.hasOwnProperty(unitCode)) {
            return SimpleMeteringServerPlugin.UNITS[unitCode];
        }
        return ['', ''];
    };
    SimpleMeteringServerPlugin.UNITS = {
        '0x00': ['kW', 'kWh'],
        '0x01': ['m3', 'm3/h'],
        '0x02': ['ft3', 'ft3/h'],
        '0x03': ['ccf', 'ccf/h'],
        '0x04': ['US gl', 'US gl/h'],
        '0x05': ['IMP gl', 'IMP gl/h'],
        '0x06': ['BTUs', 'BTU/h'],
        '0x07': ['Liters', 'l/h'],
        '0x08': ['kPA', ''],
        '0x09': ['kPA', ''],
        '0x0A': ['mcf', 'mcf/h'],
        '0x0B': ['', ''],
        '0x0C': ['MJ', 'MJ/s'],
        '0x80': ['kW', 'kWh'],
        '0x81': ['m3', 'm3/h'],
        '0x82': ['ft3', 'ft3/h'],
        '0x83': ['ccf', 'ccf/h'],
        '0x84': ['US gl', 'US gl/h'],
        '0x85': ['IMP gl', 'IMP gl/h'],
        '0x86': ['BTUs', 'BTU/h'],
        '0x87': ['Liters', 'l/h'],
        '0x88': ['kPA', ''],
        '0x89': ['kPA', ''],
        '0x8A': ['mcf', 'mcf/h'],
        '0x8B': ['', ''],
        '0x8C': ['MJ', 'MJ/s']
    };
    return SimpleMeteringServerPlugin;
}(SmartenitPlugin));
//# sourceMappingURL=simple-metering-server.plugin.js.map