import { DeviceModel } from "../models/device.model";
import { Subject, Observable } from "rxjs";
import { IWebSocketDeviceMessage } from "../websockets/websocket-device-message.interface";
import { ISmartenitPluginMessage } from "./smartenit-plugin-message.interface";
import { DatabaseService } from "../storage/database.service";
import { DevicesService } from "../resources/devices.service";
import { ActionsService } from "../resources/actions.service";
import { ConditionsService } from "../resources/conditions.service";
import { EffectsService } from "../resources/effects.service";
import { PluginUtilsService } from "./plugin-utils.service";
export declare abstract class SmartenitPlugin {
    dbService: DatabaseService;
    devicesService: DevicesService;
    actionsService: ActionsService;
    conditionsService: ConditionsService;
    effectsService: EffectsService;
    pluginUtilsService: PluginUtilsService;
    private _name;
    private _device;
    private _componentId;
    private _processorName;
    private _state;
    private _stateTTL;
    stateTTL: number;
    readonly name: string;
    readonly device: DeviceModel;
    readonly componentId: string;
    readonly processorName: string;
    state: any;
    saveState(state: any, ttl?: number): void;
    private _onData;
    readonly onData: Observable<ISmartenitPluginMessage>;
    protected _onUpdate: Subject<any>;
    readonly onUpdate: Observable<any>;
    getStatus(context?: string, force?: boolean, subscribe?: boolean, payload?: any): void;
    getStatusPayload(): any;
    getCachedValues(subscribe?: boolean): any;
    private parseCachePath(path);
    constructor(name: string, componentId: string, processorName: string, device: DeviceModel, dbService: DatabaseService, devicesService: DevicesService, actionsService: ActionsService, conditionsService: ConditionsService, effectsService: EffectsService, pluginUtilsService: PluginUtilsService);
    abstract processMessage(data: IWebSocketDeviceMessage): any;
    processDeviceMessage(data: IWebSocketDeviceMessage): any;
    getRawValue(value: any, attribute?: string, method?: string): any;
    getProcessedValue(value: any, attribute?: string, method?: string): any;
    /**
     * Executed when the plugin has loaded cache information if available
     */
    onInit(): void;
    getCacheKey(category?: string): string;
    setCache(category: string, data: any, ttl?: number, subscribe?: boolean): any;
    getCache(category?: string): Observable<any>;
    executeMethod(componentId: string, processorName: string, method: string, payload: any, requestStatus?: boolean, payloadStatus?: any, query?: string): void;
    setAttributeWithSubscribeOption(componentId: string, processorName: string, attribute: string, payload: any, subscribe: boolean, readAttribute?: boolean): Observable<any>;
    getAttributeWithSubscribeOption(componentId: string, processorName: string, attribute: string, payload: any, subscribe: boolean, query?: string): Observable<any>;
}
