import { Subject } from "rxjs";
export var SmartenitPlugin = (function () {
    function SmartenitPlugin(name, componentId, processorName, device, dbService, devicesService, actionsService, conditionsService, effectsService, pluginUtilsService) {
        var _this = this;
        this.dbService = dbService;
        this.devicesService = devicesService;
        this.actionsService = actionsService;
        this.conditionsService = conditionsService;
        this.effectsService = effectsService;
        this.pluginUtilsService = pluginUtilsService;
        this._name = name;
        this._device = device;
        this._componentId = componentId;
        this._processorName = processorName;
        this._onUpdate = new Subject();
        this._state = {};
        this._stateTTL = 10 * 60;
        device.onDeviceData
            .subscribe(function (message) {
            _this.processDeviceMessage(message);
            if (message && message.componentId === _this._componentId && message.processorName === _this._processorName) {
                _this.saveState(_this.processMessage(message));
            }
        });
        // handle api cache
        device.onDeviceResponse.subscribe(function (response) {
            if (response && response.response && response.response.apiCache == true) {
                var _a = _this.parseCachePath(response.key), componentId_1 = _a.componentId, processorName_1 = _a.processorName, attributeOrMethod = _a.attributeOrMethod;
                if (componentId_1 != undefined && processorName_1 != undefined && attributeOrMethod != undefined) {
                    if (response.response.data) {
                        // sort cache based on timestamp for consistency
                        var responseData = response.response.data;
                        responseData.sort(function (a, b) {
                            return a && a.v ? a.v.timestamp - b.v.timestamp : 0;
                        });
                        for (var i = 0; i < responseData.length; i++) {
                            var singleResponse = responseData[i];
                            if (singleResponse && singleResponse.v && singleResponse.attr) {
                                var message = {
                                    data: { response: singleResponse.v },
                                    resource: 'devices',
                                    resourceId: _this.device._id,
                                    componentId: componentId_1,
                                    processorName: processorName_1,
                                    attributeOrMethod: singleResponse.attr
                                };
                                // trigger local message to websocket to follow same process
                                device.webSocketsService.proxyMessage(message);
                            }
                        }
                    }
                }
            }
        });
        // Get cached state
        this.getCachedValues(true).subscribe(function (cachedState) {
            if (cachedState !== undefined && cachedState != null) {
                _this._state = cachedState;
            }
            _this.onInit();
        }, function () {
            _this.onInit();
        });
    }
    Object.defineProperty(SmartenitPlugin.prototype, "stateTTL", {
        get: function () {
            return this._stateTTL;
        },
        set: function (value) {
            this._stateTTL = value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SmartenitPlugin.prototype, "name", {
        get: function () {
            return this._name;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SmartenitPlugin.prototype, "device", {
        get: function () {
            return this._device;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SmartenitPlugin.prototype, "componentId", {
        get: function () {
            return this._componentId;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SmartenitPlugin.prototype, "processorName", {
        get: function () {
            return this._processorName;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SmartenitPlugin.prototype, "state", {
        get: function () {
            return this._state;
        },
        set: function (value) {
            this._state = value;
        },
        enumerable: true,
        configurable: true
    });
    SmartenitPlugin.prototype.saveState = function (state, ttl) {
        this._state = state;
        if (ttl === undefined) {
            ttl = this.stateTTL;
        }
        this.setCache('state', this._state, ttl);
    };
    Object.defineProperty(SmartenitPlugin.prototype, "onData", {
        get: function () {
            return this._onData.asObservable();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SmartenitPlugin.prototype, "onUpdate", {
        get: function () {
            return this._onUpdate.asObservable();
        },
        enumerable: true,
        configurable: true
    });
    SmartenitPlugin.prototype.getStatus = function (context, force, subscribe, payload) {
        var _this = this;
        if (payload && Object.keys(payload).length > 0) {
            payload = Object.assign(this.getStatusPayload(), payload);
        }
        else {
            payload = this.getStatusPayload();
        }
        if (force === true) {
            this._device.getStatus(this._componentId, this._processorName, payload);
        }
        else {
            this.getCachedValues(true).subscribe(function (cachedState) {
                if (cachedState == null || cachedState == undefined) {
                    _this._device.getStatus(_this._componentId, _this._processorName, payload);
                }
            });
        }
    };
    SmartenitPlugin.prototype.getStatusPayload = function () {
        return {};
    };
    SmartenitPlugin.prototype.getCachedValues = function (subscribe) {
        var _this = this;
        var observable = this.getCache().map(function (cachedState) {
            if (cachedState != null) {
                _this.state = cachedState;
                _this.device.cached = true;
                _this._onUpdate.next({
                    state: _this.state
                });
            }
            return cachedState;
        });
        if (subscribe === true) {
            return observable;
        }
        var subscription = observable.subscribe(function () { return subscription.unsubscribe(); });
    };
    SmartenitPlugin.prototype.parseCachePath = function (path) {
        if (path) {
            var parts = path.split('-');
            return { componentId: parts[0], processorName: parts[1], attributeOrMethod: parts[3] };
        }
        return null;
    };
    SmartenitPlugin.prototype.processDeviceMessage = function (data) {
    };
    SmartenitPlugin.prototype.getRawValue = function (value, attribute, method) {
        return value;
    };
    SmartenitPlugin.prototype.getProcessedValue = function (value, attribute, method) {
        return value;
    };
    /**
     * Executed when the plugin has loaded cache information if available
     */
    SmartenitPlugin.prototype.onInit = function () {
    };
    SmartenitPlugin.prototype.getCacheKey = function (category) {
        if (category === void 0) { category = ''; }
        return [this._device._id, this._componentId, this._processorName, category].join('_');
    };
    SmartenitPlugin.prototype.setCache = function (category, data, ttl, subscribe) {
        var observable = this.dbService.getCollection('devices_cache').save(this.getCacheKey(category), data, ttl);
        if (subscribe === true) {
            return observable;
        }
        return observable.subscribe(function () {
        });
    };
    SmartenitPlugin.prototype.getCache = function (category) {
        if (category === void 0) { category = 'state'; }
        return this.dbService.getCollection('devices_cache').getById(this.getCacheKey(category));
    };
    SmartenitPlugin.prototype.executeMethod = function (componentId, processorName, method, payload, requestStatus, payloadStatus, query) {
        if (requestStatus == undefined) {
            requestStatus = true;
        }
        if (requestStatus && payloadStatus == undefined) {
            payloadStatus = this.getStatusPayload();
        }
        this._device.executeMethod(componentId, processorName, method, payload, requestStatus, payloadStatus, query);
    };
    SmartenitPlugin.prototype.setAttributeWithSubscribeOption = function (componentId, processorName, attribute, payload, subscribe, readAttribute) {
        var observable = this._device.setAttribute(componentId, processorName, attribute, payload, readAttribute);
        if (subscribe === true) {
            return observable;
        }
        var subscription = observable.subscribe(function () { return subscription.unsubscribe(); });
    };
    SmartenitPlugin.prototype.getAttributeWithSubscribeOption = function (componentId, processorName, attribute, payload, subscribe, query) {
        var observable = this._device.getAttribute(componentId, processorName, attribute, payload, query);
        if (subscribe === true) {
            return observable;
        }
        var subscription = observable.subscribe(function () { return subscription.unsubscribe(); });
    };
    return SmartenitPlugin;
}());
//# sourceMappingURL=smartenit-plugin.js.map