var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
import { SmartenitPlugin } from "../smartenit-plugin";
import { ThermostatModes } from "./thermostat-modes.enum";
import { Observable } from "rxjs";
var INIT_COOLING = 10;
var INIT_HEATING = 20;
var INIT_MODE = ThermostatModes.AUTO;
var CACHE_TIME = 3600;
export var ThermostatServerPlugin = (function (_super) {
    __extends(ThermostatServerPlugin, _super);
    function ThermostatServerPlugin() {
        _super.apply(this, arguments);
    }
    ThermostatServerPlugin.prototype.setMode = function (mode, subscribe) {
        this._mode = mode;
        return this.setAttributeWithSubscribeOption(this.componentId, this.processorName, 'SystemMode', { value: mode }, subscribe);
    };
    ThermostatServerPlugin.prototype.getValue = function (attribute) {
        var value = null;
        switch (attribute) {
            case 'OccupiedCoolingSetpoint':
                value = this._desiredCoolingTemperature;
                break;
            case 'OccupiedHeatingSetpoint':
                value = this._desiredHeatingTemperature;
                break;
            case 'SystemMode':
                value = this._mode;
                break;
        }
        return value;
    };
    ThermostatServerPlugin.prototype.getUnit = function (attribute) {
        var value = '';
        switch (attribute) {
            case 'OccupiedCoolingSetpoint':
                value = 'ºC';
                break;
            case 'OccupiedHeatingSetpoint':
                value = 'ºC';
                break;
            case 'LocalTemperature':
                value = 'ºC';
                break;
            case 'SystemMode':
                value = '';
                break;
        }
        return value;
    };
    ThermostatServerPlugin.prototype.getTextValue = function () {
        var temperatureUnit = 'ºC';
        var temperature = Math.floor(this._temperature);
        return this._temperature ? temperature.toString() + temperatureUnit : '';
    };
    ThermostatServerPlugin.prototype.getConditionAttribute = function () {
        return 'SystemMode';
    };
    ThermostatServerPlugin.prototype.getConditionOptions = function () {
        return [
            {
                name: 'Off',
                value: 0
            },
            {
                name: 'Auto',
                value: 1,
            },
            {
                name: 'Heat',
                value: 4,
            },
            {
                name: 'Cool',
                value: 3,
            }
        ];
    };
    ThermostatServerPlugin.prototype.getEffectOptions = function () {
        return [
            {
                name: 'Off',
                value: 0
            },
            {
                name: 'Auto',
                value: 1,
            },
            {
                name: 'Heat',
                value: 4,
            },
            {
                name: 'Cool',
                value: 3,
            }
        ];
    };
    ThermostatServerPlugin.prototype.getEffectAttribute = function () {
        return 'SystemMode';
    };
    ThermostatServerPlugin.prototype.getEffectMethod = function (context) {
        return null;
    };
    ThermostatServerPlugin.prototype.setValue = function (value, attribute, subscribe) {
        switch (attribute) {
            case 'OccupiedCoolingSetpoint':
                this._desiredCoolingTemperature = value;
                this.setAttributeWithSubscribeOption(this.componentId, this.processorName, 'OccupiedCoolingSetpoint', { value: this._desiredCoolingTemperature * 100 }, subscribe);
                break;
            case 'OccupiedHeatingSetpoint':
                this._desiredHeatingTemperature = value;
                this.setAttributeWithSubscribeOption(this.componentId, this.processorName, 'OccupiedHeatingSetpoint', { value: this._desiredHeatingTemperature * 100 }, subscribe);
                break;
        }
    };
    ThermostatServerPlugin.prototype.getStatus = function (context, force, subscribe, payload) {
        var _this = this;
        if (context === 'OccupiedCoolingSetpoint') {
            this.getCache('OccupiedCoolingSetpoint').subscribe(function (cached) {
                if (cached == null) {
                    _this.getAttributeWithSubscribeOption(_this.componentId, _this.processorName, 'OccupiedCoolingSetpoint', null, false);
                }
                else {
                    _this._desiredCoolingTemperature = cached;
                    _this._onUpdate.next({ OccupiedCoolingSetpoint: _this._desiredCoolingTemperature });
                }
            });
        }
        else if (context === 'OccupiedHeatingSetpoint') {
            this.getCache('OccupiedHeatingSetpoint').subscribe(function (cached) {
                if (cached == null) {
                    _this.getAttributeWithSubscribeOption(_this.componentId, _this.processorName, 'OccupiedHeatingSetpoint', null, false);
                }
                else {
                    _this._desiredHeatingTemperature = cached;
                    _this._onUpdate.next({ OccupiedHeatingSetpoint: _this._desiredHeatingTemperature });
                }
            });
        }
        else {
            return _super.prototype.getStatus.call(this, context, subscribe, force, payload);
        }
    };
    ThermostatServerPlugin.prototype.getSelectedValues = function () {
        return [this._mode];
    };
    ThermostatServerPlugin.prototype.selectValue = function (option, subscribe) {
        this._mode = option;
        this._onUpdate.next({ mode: this._mode });
        return this.setMode(option, subscribe);
    };
    ThermostatServerPlugin.prototype.getCachedValues = function (subscribe) {
        var _this = this;
        var cacheValues = [
            this.getCache('OccupiedCoolingSetpoint'),
            this.getCache('OccupiedHeatingSetpoint'),
            this.getCache('SystemMode'),
            this.getCache()
        ];
        var observable = Observable.forkJoin(cacheValues).flatMap(function (array) {
            if (array && array[0] != null) {
                _this._desiredCoolingTemperature = array[0];
            }
            if (array && array[1] != null) {
                _this._desiredHeatingTemperature = array[1];
            }
            if (array && array[2] != null) {
                _this._mode = array[2];
            }
            if (array && array[3] != null) {
                _this._temperature = array[3];
            }
            if (_this._temperature != null) {
                _this.device.cached = true;
            }
            _this._onUpdate.next({
                SystemMode: _this._mode,
                OccupiedHeatingSetpoint: _this._desiredHeatingTemperature,
                OccupiedCoolingSetpoint: _this._desiredCoolingTemperature,
                temperature: _this._temperature
            });
            return Observable.of(_this._temperature);
        });
        if (subscribe === true) {
            return observable;
        }
        var subscription = observable.subscribe(function () { return subscription.unsubscribe(); });
    };
    ThermostatServerPlugin.prototype.getStatusPayload = function () {
        return ['LocalTemperature', 'SystemMode', 'OccupiedCoolingSetpoint', 'OccupiedHeatingSetpoint'];
    };
    ThermostatServerPlugin.prototype.processMessage = function (message) {
        var attribute = message && message.attributeOrMethod;
        if (attribute === 'LocalTemperature') {
            var response = message && message.data && message.data.response;
            this._temperature = response.value / 100;
            this.state = this._temperature;
            this._onUpdate.next({ temperature: this._temperature });
        }
        else if (attribute === 'SystemMode') {
            var response = message && message.data && message.data.response;
            this._mode = parseInt(response.value);
            this.setCache('SystemMode', this._mode, CACHE_TIME);
            this._onUpdate.next({ SystemMode: this._mode });
        }
        else if (attribute === 'OccupiedCoolingSetpoint') {
            var response = message && message.data && message.data.response;
            this._desiredCoolingTemperature = response.value / 100;
            this.setCache('OccupiedCoolingSetpoint', this._desiredCoolingTemperature, CACHE_TIME);
            this._onUpdate.next({ OccupiedCoolingSetpoint: this._desiredCoolingTemperature });
        }
        else if (attribute === 'OccupiedHeatingSetpoint') {
            var response = message && message.data && message.data.response;
            this._desiredHeatingTemperature = response.value / 100;
            this.setCache('OccupiedHeatingSetpoint', this._desiredHeatingTemperature, CACHE_TIME);
            this._onUpdate.next({ OccupiedHeatingSetpoint: this._desiredHeatingTemperature });
        }
        return this.state;
    };
    ThermostatServerPlugin.prototype.getRawValue = function (value, attribute, method) {
        if (!attribute || attribute === 'LocalTemperature' ||
            attribute === 'OccupiedCoolingSetpoint' || attribute === 'OccupiedHeatingSetpoint') {
            return this.pluginUtilsService.getValueByMultiplying(value, 100);
        }
        else {
            return value;
        }
    };
    ThermostatServerPlugin.prototype.getProcessedValue = function (value, attribute, method) {
        if (!attribute || attribute === 'LocalTemperature' ||
            attribute === 'OccupiedCoolingSetpoint' || attribute === 'OccupiedHeatingSetpoint') {
            return this.pluginUtilsService.getValueByDividing(value, 100);
        }
        else {
            return value;
        }
    };
    return ThermostatServerPlugin;
}(SmartenitPlugin));
//# sourceMappingURL=thermostat-server.plugin.js.map