var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
import { Injectable } from '@angular/core';
import { AuthService } from "../auth/auth.service";
import { EventsManagerService } from "../common/events-manager.service";
import { DataQueryService } from "../common/data-query.service";
import { DatabaseService } from "../storage/database.service";
import { PersistentCRUDService } from "../storage/persistent-crud.service";
import { SyncService } from "../storage/sync.service";
import { WebSocketsService } from "../websockets/websockets.service";
import { PluginFactoryService } from "../plugins/plugin-factory.service";
import { DeviceModel } from "../models/device.model";
import { HttpInterceptor } from "../common/http-interceptor.service";
export var DevicesService = (function (_super) {
    __extends(DevicesService, _super);
    function DevicesService(http, authService, dbService, webSocketsService, pluginFactory, syncService, dataQueryService, eventsService) {
        _super.call(this, 'devices', http, authService, dbService, syncService, dataQueryService, eventsService);
        this.dbService = dbService;
        this.webSocketsService = webSocketsService;
        this.pluginFactory = pluginFactory;
    }
    DevicesService.prototype.discover = function (discoverOnlyNewDevices) {
        var _this = this;
        var payload = {};
        if (discoverOnlyNewDevices === true) {
            payload.onlyNewDevices = true;
        }
        return this.post('discover', payload).map(function (apiResponse) {
            if (apiResponse && apiResponse.data && Array.isArray(apiResponse.data)) {
                for (var i = 0; i < apiResponse.data.length; i++) {
                    apiResponse.data[i] = _this.createModel(apiResponse.data[i]);
                }
            }
            return apiResponse;
        });
    };
    DevicesService.prototype.link = function (deviceId) {
        return this.post('link', { deviceId: deviceId });
    };
    DevicesService.prototype.linkDevice = function (device) {
        return this.post('link', device);
    };
    DevicesService.prototype.createModel = function (data) {
        if (data.hasOwnProperty('_id') && DevicesService._devices.hasOwnProperty(data._id)) {
            var memoryDevice = DevicesService._devices[data._id];
            memoryDevice.data = data;
            memoryDevice.loadBasicInfo(memoryDevice, data);
            memoryDevice.loadPluginsAndProcessors(memoryDevice, data);
            DevicesService._devices[data._id] = memoryDevice;
            return DevicesService._devices[data._id];
        }
        else {
            var newDevice = new DeviceModel(data, this, this.dbService, this.webSocketsService, this.pluginFactory);
            newDevice.data = data;
            DevicesService._devices[newDevice._id] = newDevice;
            return newDevice;
        }
    };
    DevicesService.prototype.removeFromLocalStorage = function (resourceId, removeData) {
        var _this = this;
        return _super.prototype.removeFromLocalStorage.call(this, resourceId, removeData)
            .map(function (removeDataRes) {
            var deviceToRemove = DevicesService._devices[resourceId];
            if (deviceToRemove) {
                deviceToRemove.clearResources();
                delete DevicesService._devices[resourceId];
            }
            _this.pluginFactory.removePluginsForDevice(resourceId);
            return removeDataRes;
        });
    };
    DevicesService.prototype.clearDevices = function () {
        Object.keys(DevicesService._devices).forEach(function (key) {
            DevicesService._devices[key].clearResources();
        });
        DevicesService._devices = {};
    };
    DevicesService._devices = {};
    DevicesService = __decorate([
        Injectable(), 
        __metadata('design:paramtypes', [HttpInterceptor, AuthService, DatabaseService, WebSocketsService, PluginFactoryService, SyncService, DataQueryService, EventsManagerService])
    ], DevicesService);
    return DevicesService;
}(PersistentCRUDService));
//# sourceMappingURL=devices.service.js.map