var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
import { Injectable } from '@angular/core';
import { APIClientService } from '../common/api-client.service';
import { AuthService } from "../auth/auth.service";
import { EventsManagerService } from "../common/events-manager.service";
import { Observable } from "rxjs";
import { HttpInterceptor } from "../common/http-interceptor.service";
export var Oauth2Service = (function (_super) {
    __extends(Oauth2Service, _super);
    function Oauth2Service(http, authService, eventsService) {
        var _this = this;
        _super.call(this, 'oauth2', http, authService, eventsService);
        authService.tokenWillExpire.subscribe(function () {
            var subscription = _this.refreshToken().subscribe(function () {
                subscription.unsubscribe();
            });
        });
    }
    Oauth2Service.prototype.logout = function () {
        var _this = this;
        this.authService.logout();
        setTimeout(function () {
            _this.eventsManagerService.trigger(EventsManagerService.ON_USER_SIGN_OUT);
        }, 1000);
    };
    Oauth2Service.prototype.login = function (username, password) {
        var _this = this;
        var clientId = this.authService.getClientId();
        var clientSecret = this.authService.getClientSecret();
        if (!clientId || !clientSecret) {
            return Observable.throw(new Error('Undefined clientId and clientSecret in AuthService'));
        }
        return this.post('token', {
            grant_type: 'password',
            username: username,
            password: password,
            client_id: clientId,
            client_secret: clientSecret
        }, { credentials: false }).map(function (response) {
            if (response) {
                _this.authService.setAccessToken(response.access_token, response.expires_in);
                _this.authService.setRefreshToken(response.refresh_token);
                _this.authService.setUser(response.data.user);
                _this.authService.setAccount(response.data.account);
                _this.eventsManagerService.trigger(EventsManagerService.ON_USER_SIGN_IN, response.data.user);
            }
            return response;
        });
    };
    Oauth2Service.prototype.authenticateClient = function (clientId, clientSecret) {
        var _this = this;
        return this.post('token', {
            grant_type: 'client_credentials',
            client_id: clientId,
            client_secret: clientSecret
        }, { credentials: false }).map(function (response) {
            if (response) {
                _this.authService.setAccessToken(response.access_token, response.expires_in);
            }
        });
    };
    Oauth2Service.prototype.refreshToken = function () {
        var _this = this;
        console.log('Refreshing Token...');
        var refreshToken = this.authService.getRefreshToken();
        if (refreshToken) {
            this.eventsManagerService.trigger(EventsManagerService.ON_TOKEN_REFRESH_START);
            var requestData = {
                grant_type: 'refresh_token',
                refresh_token: refreshToken
            };
            return this.post('token', requestData)
                .do(function (response) {
                console.log('[SDK Refresh Token] token refreshed');
                _this.authService.setAccessToken(response.access_token, response.expires_in);
                _this.eventsManagerService.trigger(EventsManagerService.ON_TOKEN_REFRESHED, { status: true, newAccessToken: 'Bearer ' + response.access_token });
            }, function (error) {
                _this.eventsManagerService.trigger(EventsManagerService.ON_TOKEN_REFRESHED, { status: false });
                console.log('[SDK refreshToken error] Couldnt refresh token', error);
                _this.eventsManagerService.trigger(EventsManagerService.ON_TOKEN_REFRESH_ERROR, error);
            });
        }
        else {
            this.eventsManagerService.trigger(EventsManagerService.ON_TOKEN_REFRESH_ERROR);
            console.log('[SDK refreshToken error] Couldnt refresh token');
            return Observable.empty();
        }
    };
    ;
    Oauth2Service = __decorate([
        Injectable(), 
        __metadata('design:paramtypes', [HttpInterceptor, AuthService, EventsManagerService])
    ], Oauth2Service);
    return Oauth2Service;
}(APIClientService));
//# sourceMappingURL=oauth-2.service.js.map