import * as localforage from "localforage";
import { Observable } from "rxjs";
export var DatabaseCollection = (function () {
    function DatabaseCollection(name) {
        this.name = name;
        this._instance = localforage.createInstance({
            name: name
        });
    }
    DatabaseCollection.prototype.getById = function (id) {
        var _this = this;
        return Observable.fromPromise(this._instance.getItem(id).then(function (item) {
            if (item && item.hasOwnProperty('ttl') && item.ttl < Date.now()) {
                _this._instance.removeItem(id).then(function () {
                    return Promise.resolve(null);
                });
            }
            else {
                return Promise.resolve(item && item.value);
            }
        }));
    };
    DatabaseCollection.generateId = function (name) {
        var buf = [];
        var chars = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789';
        var charLen = chars.length;
        for (var i = 0; i < 24; ++i)
            buf.push(chars[Math.floor(Math.random() * (charLen + 1))]);
        return 'loc_' + buf.join('') + '_' + name;
    };
    DatabaseCollection.prototype.save = function (key, value, ttl) {
        var _this = this;
        if (!key || key === undefined || key == null) {
            key = DatabaseCollection.generateId(this.name);
        }
        if (value && typeof value.toStoreObject === 'function') {
            value = value.toStoreObject();
        }
        var storeObject = { key: key, value: value };
        if (ttl && ttl != -1) {
            storeObject.ttl = Date.now() + (ttl * 1000);
        }
        return this.getById(key).map(function (element) {
            var storedItem = {};
            if (element) {
                var keys = Object.keys(storeObject);
                for (var i = 0; i < keys.length; i++) {
                    var keyValue = keys[i];
                    storedItem[keyValue] = storeObject[keyValue];
                }
            }
            else {
                storedItem = value;
            }
            _this._instance.setItem(key, storeObject);
            return storedItem;
        });
    };
    DatabaseCollection.prototype.saveLocal = function (key, value) {
        var _this = this;
        var isNew = false;
        if (!key || key === undefined || key == null) {
            key = DatabaseCollection.generateId(this.name);
            isNew = true;
        }
        if (value && typeof value.toStoreObject === 'function') {
            value = value.toStoreObject();
        }
        var storeObject = {
            key: key,
            value: Object.assign({}, value, { _id: key }),
            isNew: isNew
        };
        return this.getById(key).map(function (element) {
            if (element) {
                storeObject.value = Object.assign({}, element, storeObject.value);
            }
            _this._instance.setItem(key, storeObject);
            return storeObject;
        });
    };
    DatabaseCollection.prototype.list = function () {
        var data = [];
        return Observable.fromPromise(this._instance.iterate(function (value) {
            data.push(value);
        }).then(function () {
            return data;
        }));
    };
    DatabaseCollection.prototype.removeById = function (id) {
        return Observable.fromPromise(this._instance.removeItem(id).then(function () {
            return Promise.resolve();
        }));
    };
    DatabaseCollection.prototype.clear = function () {
        this._instance.clear();
    };
    return DatabaseCollection;
}());
//# sourceMappingURL=database-collection.js.map