var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
import { Injectable } from '@angular/core';
import { Observable } from "rxjs";
import { AuthService } from "../auth/auth.service";
import { EventsManagerService } from "../common/events-manager.service";
import { AppConfiguration, BACKEND_DATA_LIMIT } from '../common/app-configuration';
import { CRUDService } from "../common/crud.service";
import { DataQueryService } from "../common/data-query.service";
import { DatabaseService } from "./database.service";
import { SyncService } from "./sync.service";
import { HttpInterceptor } from "../common/http-interceptor.service";
export var PersistentCRUDService = (function (_super) {
    __extends(PersistentCRUDService, _super);
    function PersistentCRUDService(resource, http, authService, dbService, syncService, dataQueryService, eventsService) {
        _super.call(this, resource, http, authService, eventsService);
        this.syncService = syncService;
        this.dataQueryService = dataQueryService;
        this.eventsService = eventsService;
        this.resource = resource;
        this.collection = dbService.getCollection(resource);
    }
    PersistentCRUDService.prototype.tearUpListAll = function () {
        var _this = this;
        this.listAllObservable = this.retrieveAll()
            .flatMap(function (resourcesList) { return _this.processListData({ data: resourcesList }); })
            .share();
    };
    PersistentCRUDService.prototype.tearDownListAll = function () {
        this.listAllObservable = null;
    };
    PersistentCRUDService.prototype.save = function (data, options) {
        var _this = this;
        if (AppConfiguration.currentConfig.offlineMode && AppConfiguration.currentConfig.useOfflineOperations) {
            return this.syncService.processOfflineSave(this.resource, data);
        }
        else {
            return _super.prototype.save.call(this, data, options).flatMap(function (apiData) {
                return _this.collection.save(apiData.data._id, apiData.data).flatMap(function () {
                    return Observable.of(apiData);
                });
            });
        }
    };
    PersistentCRUDService.prototype.list = function (query, options) {
        var _this = this;
        return !this.syncService.canUseLocalStorage()
            ? (options && options.page != null) ? this.listFromBackend(query, options) : this.listAllFromBackend(false, query, options)
            : this.syncService.hasSynchronized(this.resource)
                .flatMap(function (hasSinchronized) { return hasSinchronized
                ? _this.listFromLocalStorage(query, options)
                : _this.syncService.syncFromBeggining(_this.resource).flatMap(function () {
                    return _this.listFromLocalStorage(query, options);
                }); });
    };
    PersistentCRUDService.prototype.listFromLocalStorage = function (query, options) {
        var _this = this;
        return this.collection.list().map(function (resourcesList) { return ({
            message: 'Read list from local DB',
            data: _this.dataQueryService
                .filterAndSliceData(resourcesList.map(function (data) { return _this.createModel(data.value); }), query, options),
            total: resourcesList.length
        }); });
    };
    PersistentCRUDService.prototype.listFromBackend = function (query, options) {
        var _this = this;
        return _super.prototype.list.call(this, query, options).flatMap(function (resourcesList) { return _this.processListData(resourcesList); });
    };
    PersistentCRUDService.prototype.listAllFromBackend = function (shared, query, options) {
        var _this = this;
        return shared
            ? this.listAllObservable
            : this.retrieveAll(1, query, options)
                .flatMap(function (resourcesList) { return _this.processListData({ data: resourcesList }); });
    };
    PersistentCRUDService.prototype.retrieveAll = function (page, query, options, acc) {
        var _this = this;
        if (page === void 0) { page = 1; }
        if (query === void 0) { query = null; }
        if (options === void 0) { options = {}; }
        if (acc === void 0) { acc = []; }
        return Observable.defer(function () { return _super.prototype.list.call(_this, query, Object.assign({}, options, { page: page }))
            .flatMap(function (_a) {
            var data = _a.data, _links = _a._links;
            acc = acc.concat(data);
            var items = Observable.of(acc);
            var nextData = (_links && ((page + 1) <= Math.ceil(_links.total / BACKEND_DATA_LIMIT)))
                ? _this.retrieveAll(page + 1, query, options, acc)
                : Observable.empty();
            return Observable.concat(items, nextData);
        }); });
    };
    PersistentCRUDService.prototype.processListData = function (resourcesList) {
        var saveOperations = [];
        for (var i = 0; i < resourcesList.data.length; i++) {
            var resourceObject = resourcesList.data[i];
            if (resourceObject && resourceObject._id) {
                saveOperations.push(this.collection.save(resourceObject._id, resourceObject));
            }
        }
        if (saveOperations.length > 0) {
            return Observable.forkJoin(saveOperations).flatMap(function (res) {
                return Observable.of(resourcesList);
            });
        }
        else {
            return Observable.of(resourcesList);
        }
    };
    PersistentCRUDService.prototype.getById = function (resourceId, options) {
        var _this = this;
        return !this.syncService.canUseLocalStorage()
            ? this.getByIdFromBackend(resourceId, options)
            : this.syncService.hasSynchronized(this.resource)
                .flatMap(function (hasSinchronized) { return hasSinchronized
                ? _this.getByIdFromLocalStorage(resourceId, options)
                : _this.syncService.syncFromBeggining(_this.resource).flatMap(function () {
                    return _this.getByIdFromLocalStorage(resourceId, options);
                }); });
    };
    PersistentCRUDService.prototype.getByIdFromLocalStorage = function (resourceId, options) {
        var _this = this;
        return this.collection.getById(resourceId).flatMap(function (resourceResponse) {
            if (resourceResponse) {
                return Observable.of({
                    message: 'Read item from local DB',
                    data: _this.createModel(resourceResponse)
                });
            }
            else {
                return _this.getByIdFromBackend(resourceId, options);
            }
        });
    };
    PersistentCRUDService.prototype.getByIdFromBackend = function (resourceId, options) {
        var _this = this;
        return _super.prototype.getById.call(this, resourceId, options).flatMap(function (resourceResponse) {
            return _this.collection.save(resourceId, resourceResponse.data).flatMap(function () {
                return Observable.of(resourceResponse);
            });
        });
    };
    PersistentCRUDService.prototype.remove = function (path, data, options) {
        var _this = this;
        if (AppConfiguration.currentConfig.offlineMode && AppConfiguration.currentConfig.useOfflineOperations) {
            return this.syncService.processOfflineDelete(this.resource, path);
        }
        else {
            return _super.prototype.remove.call(this, path, data, options).flatMap(function (removeData) {
                return _this.removeFromLocalStorage(path, removeData);
            });
        }
    };
    PersistentCRUDService.prototype.removeFromLocalStorage = function (resourceId, removeData) {
        return this.collection.removeById(resourceId).map(function () { return (removeData); });
    };
    PersistentCRUDService.prototype.removeById = function (resourceId, options) {
        return this.remove(resourceId, null, options);
    };
    PersistentCRUDService.prototype.mergeInCollection = function (resourceId, resource) {
        var _this = this;
        return this.collection.getById(resourceId)
            .flatMap(function (resourceResponse) {
            if (resourceResponse) {
                return _this.collection.save(resourceId, Object.assign({}, resourceResponse, resource));
            }
            else {
                return Observable.of(resourceResponse);
            }
        });
    };
    PersistentCRUDService = __decorate([
        Injectable(), 
        __metadata('design:paramtypes', [String, HttpInterceptor, AuthService, DatabaseService, SyncService, DataQueryService, EventsManagerService])
    ], PersistentCRUDService);
    return PersistentCRUDService;
}(CRUDService));
//# sourceMappingURL=persistent-crud.service.js.map