import { Injector } from '@angular/core';
import { Observable, Subscription, Subject } from 'rxjs';
import 'rxjs/add/observable/throw';
import { AuthService } from '../auth/auth.service';
import { EventsManagerService } from "../common/events-manager.service";
import { APIClientService } from '../common/api-client.service';
import { DataQueryService } from '../common/data-query.service';
import { WebSocketsService } from '../websockets/websockets.service';
import { DatabaseCollection } from './database-collection';
import { DatabaseService } from './database.service';
import { HttpInterceptor } from "../common/http-interceptor.service";
export declare class SyncService extends APIClientService {
    private webSocketsService;
    private dbService;
    private dataQueryService;
    private injector;
    eventsService: EventsManagerService;
    useLocalStorage: boolean;
    useOfflineOperations: boolean;
    syncCollection: DatabaseCollection;
    syncSubscription: Subscription;
    offlineCollection: DatabaseCollection;
    syncChangeSubject: Subject<string>;
    readonly onSyncChange: Observable<string>;
    constructor(http: HttpInterceptor, authService: AuthService, webSocketsService: WebSocketsService, dbService: DatabaseService, dataQueryService: DataQueryService, injector: Injector, eventsService: EventsManagerService);
    /**
     * @private
     * Process the sync events received asynchronously
     */
    private configureEventsListener();
    /**
     * @public
     * Start the synchronization timeout process
     */
    startSync(): void;
    /**
     * @public
     * Stop the synchronization timeout process
     */
    stopSync(): void;
    /**
     * @private
     * Retrieves resources available for sync changes
     */
    private getSyncInfo();
    /**
     * @private
     * Request and process the changes for a resource since a given date
     * @param resource Resource name
     * @param lastSyncDate Last date of synchronization
     */
    private syncResource(resource, lastSyncDate);
    /**
     * @private
     * Retrieves all changes for a resource since a given date
     * @param resource Resource name
     * @param lastSyncDate Last date of synchronization
     * @param page Current page of total data
     */
    private retrieveAllChanges(resource, lastSyncDate, page?);
    /**
     * @private
     * Process a single synchronization change
     * @param operation Operation type (save/delete)
     * @param resource Name of the resource
     * @param resourceId Id of the resource
     * @param resourceService Service associated with the resource
     */
    private processSyncChange(operation, resource, resourceId, resourceService?);
    /**
     * @public
     * Sync all data for the resource
     * @param resource Resource name
     */
    syncFromBeggining(resource: string): Observable<any>;
    /**
     * @public
     * Indicates if the use of local storage is enabled
     */
    canUseLocalStorage(): boolean;
    /**
     * @public
     * Indicates if this resource has been
     * sinchronized previously
     * @param resource Resource name
     */
    hasSynchronized(resource: string): Observable<boolean>;
    /**
     * @public
     * Synchronize all operations made in offline mode
     */
    syncOfflineOperations(): void;
    private syncOfflineOperation(key, value);
    private syncOfflineSave(key, saveOperation);
    private syncOfflineUpdate(key, updateOperation);
    private syncOfflineDelete(key, deleteOperation);
    /**
     * Update all the id references in the offline operations
     * @param newId Id retrieved from the server
     * @param oldId Id generated locally
     */
    private updateLocalIdReferences(resourceId, newId, oldId);
    /**
     * @public
     * Process an offline save operation record
     * @param resource Name of the resource
     * @param data Data associated with the save operation
     */
    processOfflineSave(resource: string, data: any): Observable<any>;
    private addOfflineSave(resource, data);
    /**
     * @public
     * Process an offline delete operation record
     * @param resource Name of the resource
     * @param resourceId Id of the resource
     */
    processOfflineDelete(resource: string, resourceId: string): Observable<any>;
    private addOfflineDelete(resource, resourceId);
}
