var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
import { Injectable, Injector } from '@angular/core';
import { Observable, Subject } from 'rxjs';
import 'rxjs/add/observable/throw';
import { AuthService } from '../auth/auth.service';
import { EventsManagerService } from "../common/events-manager.service";
import { AppConfiguration, BACKEND_DATA_LIMIT } from '../common/app-configuration';
import { APIClientService } from '../common/api-client.service';
import { DataQueryService } from '../common/data-query.service';
import { changeValueInObject } from '../common/utils';
import { ResourcesProvider } from '../resources/resources.provider';
import { WebSocketsService } from '../websockets/websockets.service';
import { DatabaseService } from './database.service';
import { OfflineOperation } from './sync-offline-operation.interface';
import { HttpInterceptor } from "../common/http-interceptor.service";
var SYNC_NAME = 'sync';
var OFFLINE_NAME = 'offline';
var SYNC_TIMEOUT = 5 * 60 * 1000;
export var SyncService = (function (_super) {
    __extends(SyncService, _super);
    function SyncService(http, authService, webSocketsService, dbService, dataQueryService, injector, eventsService) {
        _super.call(this, SYNC_NAME, http, authService, eventsService);
        this.webSocketsService = webSocketsService;
        this.dbService = dbService;
        this.dataQueryService = dataQueryService;
        this.injector = injector;
        this.eventsService = eventsService;
        this.syncChangeSubject = new Subject();
        this.useLocalStorage = AppConfiguration.currentConfig.useLocalStorage;
        this.useOfflineOperations = AppConfiguration.currentConfig.useOfflineOperations;
        this.syncCollection = dbService.getCollection(SYNC_NAME);
        this.offlineCollection = dbService.getCollection(OFFLINE_NAME);
        this.configureEventsListener();
    }
    Object.defineProperty(SyncService.prototype, "onSyncChange", {
        get: function () {
            return this.syncChangeSubject.asObservable();
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @private
     * Process the sync events received asynchronously
     */
    SyncService.prototype.configureEventsListener = function () {
        var _this = this;
        if (!this.useLocalStorage)
            return;
        this.webSocketsService.onEventsMessage
            .filter(function (msg) {
            return msg && msg.data && msg.data.response &&
                msg.data.response.code &&
                msg.data.response.identityType === 'sync' &&
                msg.data.response.resourcePath && msg.data.response.resourcePath.split('/').length >= 3;
        })
            .map(function (msg) {
            var response = msg.data.response;
            return {
                operation: response.code.split('.').pop(),
                resource: response.resourcePath.split('/')[1],
                resourceId: response.resourcePath.split('/')[2],
                date: (new Date()).toISOString()
            };
        })
            .flatMap(function (event) {
            return _this.hasSynchronized(event.resource)
                .map(function (hasSinchronized) { return ({
                hasSinchronized: hasSinchronized,
                event: event
            }); });
        })
            .filter(function (resEvent) { return resEvent.hasSinchronized; })
            .flatMap(function (_a) {
            var event = _a.event;
            console.log(event);
            return _this.processSyncChange(event.operation, event.resource, event.resourceId)
                .catch(function () { return Observable.empty(); })
                .map(function () { return event; });
        })
            .subscribe(function (event) {
            if (event.resource) {
                _this.syncChangeSubject.next(event.resource);
                var syncSub_1 = _this.syncCollection
                    .save(event.resource, { updatedAt: event.date })
                    .subscribe(function () { syncSub_1.unsubscribe(); });
            }
        });
    };
    /**
     * @public
     * Start the synchronization timeout process
     */
    SyncService.prototype.startSync = function () {
        var _this = this;
        if (!this.useLocalStorage || this.syncSubscription)
            return;
        this.syncSubscription = Observable.of('')
            .flatMap(function () { return _this.getSyncInfo().delay(SYNC_TIMEOUT); })
            .repeat()
            .subscribe(function () { });
    };
    /**
     * @public
     * Stop the synchronization timeout process
     */
    SyncService.prototype.stopSync = function () {
        if (!this.useLocalStorage || !this.syncSubscription)
            return;
        console.log('Stoping synchronization');
        this.syncSubscription.unsubscribe();
        this.syncSubscription = null;
    };
    /**
     * @private
     * Retrieves resources available for sync changes
     */
    SyncService.prototype.getSyncInfo = function () {
        var _this = this;
        if (!this.useLocalStorage || AppConfiguration.currentConfig.offlineMode)
            return Observable.create(function (o) { return o.complete(); });
        console.log('Starting synchronization');
        return _super.prototype.get.call(this).map(function (res) {
            _this.syncCollection.list().subscribe(function (syncRes) {
                if (syncRes && syncRes.length) {
                    if (res.data) {
                        Object.keys(res.data).forEach(function (key) {
                            if (res.data[key] && res.data[key].hasOwnProperty('updatedAt')) {
                                var lastServerSyncDate = new Date(res.data[key].updatedAt);
                                var syncResItem = syncRes.find(function (sr) { return sr.key === key && sr.value && sr.value.hasOwnProperty('updatedAt'); });
                                if (syncResItem) {
                                    var lastLocalSyncDate = new Date(syncResItem.value.updatedAt);
                                    if (lastLocalSyncDate.getTime() < lastServerSyncDate.getTime()) {
                                        _this.syncResource(key, lastLocalSyncDate).subscribe(function () { });
                                    }
                                }
                            }
                        });
                    }
                    else {
                        console.log('Invalid sync info from server');
                    }
                }
                else {
                    console.log('No previous data sync, backend query needed');
                }
            }, function (error) {
                console.log('Error getting local sync collection', error);
            });
        }, function (error) {
            console.log('Error getting sync info', error);
        });
    };
    /**
     * @private
     * Request and process the changes for a resource since a given date
     * @param resource Resource name
     * @param lastSyncDate Last date of synchronization
     */
    SyncService.prototype.syncResource = function (resource, lastSyncDate) {
        var _this = this;
        if (!this.useLocalStorage || AppConfiguration.currentConfig.offlineMode)
            return;
        var resourceService = this.injector.get(ResourcesProvider).getResourceToken(resource);
        console.log("Synchronizing " + resource);
        return this.retrieveAllChanges(resource, lastSyncDate)
            .flatMap(function (data) {
            return data.map(function (change) {
                if (change && change.operation) {
                    return _this.processSyncChange(change.operation, resource, change.resourceId, resourceService);
                }
                else {
                    return Observable.empty();
                }
            }).reduce(function (acc, curr) {
                return acc.flatMap(function () { return curr; });
            }).map(function () {
                return data.length && data[data.length - 1].hasOwnProperty('updatedAt')
                    ? data[data.length - 1].updatedAt
                    : (new Date()).toISOString();
            });
        })
            .catch(function (error) {
            console.log("Error getting sync info for resource " + resource, error);
            if (error && error.status === 404) {
                console.log('Sync info not found, downloading all info');
                return resourceService.listAllFromBackend(true).map(function () { return (new Date()).toISOString(); });
            }
            return Observable.empty();
        })
            .flatMap(function (updatedAt) {
            _this.syncChangeSubject.next(resource);
            return _this.syncCollection
                .save(resource, { updatedAt: updatedAt })
                .flatMap(function () {
                return Observable.of(updatedAt);
            });
        });
    };
    /**
     * @private
     * Retrieves all changes for a resource since a given date
     * @param resource Resource name
     * @param lastSyncDate Last date of synchronization
     * @param page Current page of total data
     */
    SyncService.prototype.retrieveAllChanges = function (resource, lastSyncDate, page) {
        var _this = this;
        if (page === void 0) { page = 1; }
        return Observable.defer(function () { return _super.prototype.get.call(_this, null, {
            resource: resource,
            lastSyncAt: lastSyncDate.toISOString(),
            page: page
        })
            .flatMap(function (_a) {
            var data = _a.data, _links = _a._links;
            var items = Observable.of(data);
            var nextData = (_links && ((page + 1) <= Math.ceil(_links.total / BACKEND_DATA_LIMIT)))
                ? _this.retrieveAllChanges(resource, lastSyncDate, page + 1)
                : Observable.empty();
            return Observable.concat(items, nextData);
        }); });
    };
    /**
     * @private
     * Process a single synchronization change
     * @param operation Operation type (save/delete)
     * @param resource Name of the resource
     * @param resourceId Id of the resource
     * @param resourceService Service associated with the resource
     */
    SyncService.prototype.processSyncChange = function (operation, resource, resourceId, resourceService) {
        var service = !resourceService
            ? this.injector.get(ResourcesProvider).getResourceToken(resource)
            : resourceService;
        switch (operation.toLowerCase()) {
            case 'save':
                return service.getByIdFromBackend(resourceId).catch(function () { return Observable.empty(); });
            case 'delete':
                return service.removeFromLocalStorage(resourceId, {});
            default:
                return Observable.empty();
        }
    };
    /**
     * @public
     * Sync all data for the resource
     * @param resource Resource name
     */
    SyncService.prototype.syncFromBeggining = function (resource) {
        var _this = this;
        var resourceService = this.injector.get(ResourcesProvider).getResourceToken(resource);
        return resourceService.listAllFromBackend(false).flatMap(function (data) {
            return _this.syncCollection.save(resource, { updatedAt: (new Date()).toISOString() })
                .flatMap(function () {
                return Observable.of(data);
            });
        });
    };
    /**
     * @public
     * Indicates if the use of local storage is enabled
     */
    SyncService.prototype.canUseLocalStorage = function () {
        return this.useLocalStorage;
    };
    /**
     * @public
     * Indicates if this resource has been
     * sinchronized previously
     * @param resource Resource name
     */
    SyncService.prototype.hasSynchronized = function (resource) {
        return !this.useLocalStorage
            ? Observable.of(false)
            : this.syncCollection
                .getById(resource)
                .flatMap(function (syncRes) { return Observable.of(!!syncRes && syncRes.hasOwnProperty('updatedAt')); });
    };
    /**
     * @public
     * Synchronize all operations made in offline mode
     */
    SyncService.prototype.syncOfflineOperations = function () {
        var _this = this;
        if (!this.useOfflineOperations || AppConfiguration.currentConfig.offlineMode)
            return;
        console.log('Synchronizing offline data');
        this.offlineCollection.list()
            .subscribe(function (operations) {
            var obs = Observable.of('');
            operations.sort(function (o1, o2) { return new Date(o1.value.date) < new Date(o2.value.date) ? -1 : 1; });
            operations.forEach(function (operation) {
                obs = obs.flatMap(function () {
                    return _this.offlineCollection.getById(operation.key)
                        .flatMap(function (value) { return _this.syncOfflineOperation(operation.key, value); });
                });
            });
            obs.subscribe(function () {
                console.log('Offline synchronization finished');
            }, function () {
                console.log('Offline synchronization finished with errors');
            });
        });
    };
    SyncService.prototype.syncOfflineOperation = function (key, value) {
        var _this = this;
        var resourceIdObservable;
        switch (value.operation) {
            case OfflineOperation.SAVE:
                resourceIdObservable = this.syncOfflineSave(key, value);
                break;
            case OfflineOperation.UPDATE:
                resourceIdObservable = this.syncOfflineUpdate(key, value);
                break;
            case OfflineOperation.DELETE:
                resourceIdObservable = this.syncOfflineDelete(key, value);
                break;
        }
        return resourceIdObservable
            .catch(function (err) {
            console.log('Error synchronizing offline resource:', value.resource, err);
            return Observable.throw(err);
        })
            .flatMap(function (resourceId) {
            return _this.offlineCollection.removeById(key)
                .flatMap(function () {
                _this.syncChangeSubject.next(value.resource);
                return Observable.of(resourceId);
            });
        });
    };
    SyncService.prototype.syncOfflineSave = function (key, saveOperation) {
        var _this = this;
        var resourceService = this.injector.get(ResourcesProvider).getResourceToken(saveOperation.resource);
        return resourceService.save(saveOperation.model)
            .flatMap(function (apiData) {
            return ((saveOperation.resourceId.startsWith('loc_'))
                ? resourceService.collection.removeById(saveOperation.resourceId)
                    .flatMap(function () { return _this.updateLocalIdReferences(key, apiData.data._id, saveOperation.resourceId); })
                : Observable.of(''))
                .catch(function (err) {
                console.log('error synchronizing save for resource:', saveOperation.resourceId, err);
                return Observable.empty();
            })
                .flatMap(function () {
                return Observable.of(saveOperation.resourceId);
            });
        })
            .catch(function (err) {
            if (err.status === 400) {
                // Dependent resource does not exists then discard operation
                return Observable.of(saveOperation.resourceId);
            }
            return Observable.throw(err);
        });
    };
    SyncService.prototype.syncOfflineUpdate = function (key, updateOperation) {
        var resourceService = this.injector.get(ResourcesProvider).getResourceToken(updateOperation.resource);
        return resourceService.save(Object.assign({}, updateOperation.model, { changedLocally: true, updatedAt: updateOperation.date }))
            .catch(function (err) {
            console.log('error synchronizing update for resource:', updateOperation.resourceId, err);
            if (err.status === 409) {
                return resourceService.collection.save(updateOperation.resourceId, err.data.data);
            }
            else if (err.status === 404) {
                // Resource does not exists then discard operation
                return Observable.of(updateOperation.resourceId);
            }
            return Observable.empty();
        })
            .flatMap(function () {
            return Observable.of(updateOperation.resourceId);
        });
    };
    SyncService.prototype.syncOfflineDelete = function (key, deleteOperation) {
        var resourceService = this.injector.get(ResourcesProvider).getResourceToken(deleteOperation.resource);
        return resourceService.remove(deleteOperation.resourceId, { changedLocally: true, updatedAt: deleteOperation.date })
            .catch(function (err) {
            console.log('error synchronizing deletion for resource:', deleteOperation.resourceId, err);
            if (err.status === 409) {
                return resourceService.collection.save(deleteOperation.resourceId, err.data.data);
            }
            else if (err.status === 404) {
                // Resource does not exists then discard operation
                return Observable.of(deleteOperation.resourceId);
            }
            return Observable.empty();
        })
            .flatMap(function () {
            return Observable.of(deleteOperation.resourceId);
        });
    };
    /**
     * Update all the id references in the offline operations
     * @param newId Id retrieved from the server
     * @param oldId Id generated locally
     */
    SyncService.prototype.updateLocalIdReferences = function (resourceId, newId, oldId) {
        var _this = this;
        return this.offlineCollection.list()
            .flatMap(function (operations) {
            return Observable.forkJoin(operations
                .filter(function (o) { return o.key !== resourceId; })
                .map(function (o) {
                changeValueInObject(o.value.model, newId, oldId);
                if (o.key.startsWith(oldId)) {
                    o.value.model._id = newId;
                }
                return _this.offlineCollection.save(o.key, o.value);
            })).defaultIfEmpty();
        });
    };
    /**
     * @public
     * Process an offline save operation record
     * @param resource Name of the resource
     * @param data Data associated with the save operation
     */
    SyncService.prototype.processOfflineSave = function (resource, data) {
        var _this = this;
        if (!this.useOfflineOperations || !AppConfiguration.currentConfig.offlineMode)
            return;
        var resourceService = this.injector.get(ResourcesProvider).getResourceToken(resource);
        return resourceService.collection.saveLocal(data._id, data).flatMap(function (storeResult) {
            return _this.addOfflineSave(resource, storeResult).flatMap(function () {
                return Observable.of({
                    data: resourceService.createModel(Object.assign({}, storeResult.value, {
                        _id: storeResult.key
                    }))
                });
            });
        });
    };
    SyncService.prototype.addOfflineSave = function (resource, data) {
        var model = Object.assign({}, data.value);
        if (data.isNew)
            delete model._id;
        return this.offlineCollection.save(data.key + "_" + new Date().getTime(), {
            operation: data.isNew ? OfflineOperation.SAVE : OfflineOperation.UPDATE,
            resource: resource,
            resourceId: data.key,
            model: model,
            date: (new Date()).toISOString()
        });
    };
    /**
     * @public
     * Process an offline delete operation record
     * @param resource Name of the resource
     * @param resourceId Id of the resource
     */
    SyncService.prototype.processOfflineDelete = function (resource, resourceId) {
        var _this = this;
        if (!this.useOfflineOperations || !AppConfiguration.currentConfig.offlineMode)
            return;
        var resourceService = this.injector.get(ResourcesProvider).getResourceToken(resource);
        return resourceService.removeFromLocalStorage(resourceId, {}).flatMap(function () {
            return _this.addOfflineDelete(resource, resourceId);
        });
    };
    SyncService.prototype.addOfflineDelete = function (resource, resourceId) {
        var _this = this;
        return this.offlineCollection.list()
            .flatMap(function (operations) {
            var existingOperations = operations.filter(function (o) { return o.key.startsWith(resourceId); });
            return Observable.forkJoin(existingOperations.map(function (o) { return _this.offlineCollection.removeById(o.key); }))
                .defaultIfEmpty()
                .flatMap(function () {
                return ((!resourceId.startsWith('loc_'))
                    ? _this.offlineCollection.save(resourceId + "_" + new Date().getTime(), {
                        operation: OfflineOperation.DELETE,
                        resource: resource,
                        resourceId: resourceId,
                        date: (new Date()).toISOString()
                    })
                    : Observable.of(''))
                    .flatMap(function () {
                    return Observable.of({
                        data: {
                            _id: resourceId
                        }
                    });
                });
            });
        });
    };
    SyncService = __decorate([
        Injectable(), 
        __metadata('design:paramtypes', [HttpInterceptor, AuthService, WebSocketsService, DatabaseService, DataQueryService, Injector, EventsManagerService])
    ], SyncService);
    return SyncService;
}(APIClientService));
//# sourceMappingURL=sync.service.js.map