var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
import { Injectable } from '@angular/core';
import { APIClientService } from '../common/api-client.service';
import { AuthService } from "../auth/auth.service";
import { EventsManagerService } from "../common/events-manager.service";
import { Oauth2Service } from "../resources/oauth-2.service";
import { Subject } from "rxjs";
import { HttpInterceptor } from "../common/http-interceptor.service";
var MAX_RETRIES = 5;
export var WebSocketsService = (function (_super) {
    __extends(WebSocketsService, _super);
    function WebSocketsService(http, authService, oauthService, eventsService) {
        var _this = this;
        _super.call(this, 'websockets', http, authService, eventsService);
        this.oauthService = oauthService;
        this.eventsService = eventsService;
        this._clientDisconnect = false;
        this._isConnected = false;
        this._onDeviceMessage = new Subject();
        this._onEventsMessage = new Subject();
        this._onScenesMessage = new Subject();
        this._onPairingsMessage = new Subject();
        this._onConnectionLost = new Subject();
        this._onConnect = new Subject();
        this._onConnectError = new Subject();
        this._onSubscribe = new Subject();
        this._onReady = new Subject();
        this.reconnectionAttempts = 0;
        this.eventsManagerService.onTokenRefreshed.subscribe(function (success) {
            if (success) {
                _this.disconnect(true);
            }
        });
        this.authService.setWebSocketsService(this);
    }
    Object.defineProperty(WebSocketsService.prototype, "onConnectionLost", {
        get: function () {
            return this._onConnectionLost.asObservable();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(WebSocketsService.prototype, "onConnect", {
        get: function () {
            return this._onConnect.asObservable();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(WebSocketsService.prototype, "onConnectError", {
        get: function () {
            return this._onConnectError.asObservable();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(WebSocketsService.prototype, "onSubscribe", {
        get: function () {
            return this._onSubscribe.asObservable();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(WebSocketsService.prototype, "onDeviceMessage", {
        get: function () {
            return this._onDeviceMessage.asObservable();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(WebSocketsService.prototype, "onScenesMessage", {
        get: function () {
            return this._onScenesMessage.asObservable();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(WebSocketsService.prototype, "onPairingsMessage", {
        get: function () {
            return this._onPairingsMessage.asObservable();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(WebSocketsService.prototype, "onEventsMessage", {
        get: function () {
            return this._onEventsMessage.asObservable();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(WebSocketsService.prototype, "onReady", {
        get: function () {
            return this._onReady.asObservable();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(WebSocketsService.prototype, "isConnected", {
        get: function () {
            return this._isConnected;
        },
        enumerable: true,
        configurable: true
    });
    WebSocketsService.prototype.connect = function () {
        var _this = this;
        this._clientDisconnect = false;
        if (!this.isConnected) {
            var subscription_1 = this.post('auth').subscribe(function (authResponse) {
                _this.connectToMQTTBroker(authResponse.hostUrl, authResponse.clientId, authResponse.topics);
                subscription_1.unsubscribe();
            });
        }
        else {
            console.error('WebSockets: already connected');
        }
    };
    WebSocketsService.prototype.disconnect = function (retryable) {
        if (retryable === void 0) { retryable = false; }
        this._clientDisconnect = !retryable;
        if (this.client && this.isConnected) {
            this.client.disconnect();
        }
    };
    WebSocketsService.prototype.send = function (topic, message) {
        if (!this.client || !this.isConnected) {
            throw new Error('WebSockets: not connected connected');
        }
        var messageObject = new Paho.MQTT.Message(JSON.stringify(message));
        messageObject.destinationName = topic;
        this.client.send(messageObject);
    };
    WebSocketsService.prototype.parseMessage = function (message) {
        var payload = {};
        if (message.payloadString && message.payloadString.length > 0) {
            try {
                payload = JSON.parse(message.payloadString);
            }
            catch (e) {
                console.error('Error parsing JSON payload from MQTT broker');
            }
        }
        var topic = message.destinationName;
        var split = topic.split('/');
        if (split && split.length >= 3) {
            var parsedMessage = {
                resource: split[3],
                resourceId: split[4],
                data: payload
            };
            if (split.length >= 5) {
                parsedMessage.componentId = split[5];
            }
            if (split.length >= 6) {
                parsedMessage.processorName = split[6];
            }
            if (split.length >= 7) {
                parsedMessage.attributeOrMethod = split[7];
            }
            return parsedMessage;
        }
        return null;
    };
    WebSocketsService.prototype.proxyMessage = function (message) {
        if (message) {
            switch (message.resource) {
                case 'devices':
                    this._onDeviceMessage.next(message);
                    break;
                case 'events':
                    this._onEventsMessage.next(message);
                    break;
                case 'scenes':
                    this._onScenesMessage.next(message);
                    break;
                case 'pairings':
                    this._onPairingsMessage.next(message);
                    break;
            }
        }
    };
    WebSocketsService.prototype.connectToMQTTBroker = function (hostUrl, clientId, topics) {
        var _this = this;
        this._isConnected = false;
        this.client = new Paho.MQTT.Client(hostUrl, clientId);
        this.client.onConnectionLost = function (responseObject) {
            _this._isConnected = false;
            _this._onConnectionLost.next(responseObject);
            if (!_this._clientDisconnect) {
                if (_this.reconnectionAttempts < MAX_RETRIES) {
                    _this.reconnectionAttempts++;
                    setTimeout(function () {
                        _this.connect();
                    }, 1000);
                }
            }
        };
        this.client.onMessageArrived = function (message) {
            var parsedMessage = _this.parseMessage(message);
            _this.proxyMessage(parsedMessage);
        };
        this.client.connect({
            useSSL: true,
            timeout: 3,
            mqttVersion: 4,
            keepAliveInterval: 1700,
            onSuccess: function () {
                _this._isConnected = true;
                _this._onConnect.next({});
                _this.reconnectionAttempts = 0;
                var _loop_1 = function(i) {
                    var topic = topics[i];
                    _this.client.subscribe(topic, {
                        onSuccess: function () {
                            _this._onSubscribe.next(topic);
                            if (i >= topics.length - 1) {
                                _this._onReady.next({});
                            }
                        }
                    });
                };
                for (var i = 0; i < topics.length; i++) {
                    _loop_1(i);
                }
            },
            onFailure: function (error) {
                _this._isConnected = false;
                _this._onConnectError.next(error);
                if (_this.reconnectionAttempts < MAX_RETRIES) {
                    _this.reconnectionAttempts++;
                    setTimeout(function () {
                        _this.connect();
                    }, 1000);
                }
            }
        });
    };
    WebSocketsService = __decorate([
        Injectable(), 
        __metadata('design:paramtypes', [HttpInterceptor, AuthService, Oauth2Service, EventsManagerService])
    ], WebSocketsService);
    return WebSocketsService;
}(APIClientService));
//# sourceMappingURL=websockets.service.js.map